@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2007 Bill Wilson
+|  Copyright (C) 1999-2010 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -17,6 +17,18 @@
 |
 |  You should have received a copy of the GNU General Public License
 |  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellmd.h"
@@ -80,16 +92,18 @@ gkrellmd_serve_data(GkrellmdMonitor *mon, gchar *line)
 		if (mp->serve_name)
 			{
 			snprintf(buf, sizeof(buf), "<%s>\n", mp->serve_name);
+			gkrellm_debug(DEBUG_SERVER, "%s", buf);
 			mp->serve_gstring = g_string_append(mp->serve_gstring, buf);
 			mp->serve_name_sent = TRUE;
 			}
 		else
 			{
-			printf("gkrellmd: %s forgot to gkrellmd_set_serve_name()\n",
+			g_warning("gkrellmd: %s forgot to gkrellmd_set_serve_name()\n",
 					mon->name);
 			return;
 			}
 		}
+	gkrellm_debug(DEBUG_SERVER,"%s", line);
 	mp->serve_gstring = g_string_append(mp->serve_gstring, line);
 	}
 
@@ -189,7 +203,7 @@ serve_cpu_data(GkrellmdMonitor *mon, gboolean first_serve)
 	for (list = cpu_list; list; list = list->next)
 		{
 		cpu = (CpuData *) list->data;
-		sprintf(buf, "%d %lu %lu %lu %lu\n", cpu->instance,
+		snprintf(buf, sizeof(buf), "%d %lu %lu %lu %lu\n", cpu->instance,
 				cpu->user, cpu->nice, cpu->sys, cpu->idle);
 		gkrellmd_serve_data(mon, buf);
 		}
@@ -335,7 +349,7 @@ static gboolean	units_are_blocks;
 
 
 static DiskData *
-add_disk(gchar *name, gint order, gint device_number, gint unit_number)
+add_disk(const gchar *name, gint order, gint device_number, gint unit_number)
 	{
 	DiskData	*disk;
 	GList		*list;
@@ -421,6 +435,19 @@ gkrellm_disk_units_are_blocks(void)
 	units_are_blocks = TRUE;
 	}
 
+void
+gkrellm_disk_add_by_name(const gchar *name, const gchar *label)
+	{
+	gint	order = -1;
+
+    if (NULL == name) // Cannot add disk without a name
+		return;
+    order = gkrellm_sys_disk_order_from_name(name);
+    /* FIXME: gkrellmd currently has no support for disk labels. Extend
+       network-protocol and server to support disks with both name and label. */
+    add_disk(name, order, 0, 0);
+	}
+
 void
 gkrellm_disk_assign_data_by_device(gint device_number, gint unit_number,
 			guint64 rb, guint64 wb, gboolean virtual)
@@ -459,7 +486,7 @@ gkrellm_disk_assign_data_nth(gint n, guint64 rb, guint64 wb, gboolean virtual)
 		disk = (DiskData *) g_list_nth_data(disk_list, n);
 	else
 		{
-		sprintf(name, "%s%c", _("Disk"), 'A' + n);
+		snprintf(name, sizeof(name), "%s%c", _("Disk"), 'A' + n);
 		disk = add_disk(name, n, 0, 0);
 		}
 	disk_assign_data(disk, rb, wb, virtual);
@@ -544,7 +571,7 @@ serve_disk_data(GkrellmdMonitor *mon, gboolean first_serve)
 	{
 	DiskData	*disk;
 	GList		*list;
-	gchar		buf[128];
+	gchar		*buf = NULL;
 
 	gkrellmd_set_serve_name(mon, "disk");
 	for (list = disk_list; list; list = list->next)
@@ -555,18 +582,20 @@ serve_disk_data(GkrellmdMonitor *mon, gboolean first_serve)
 		if (!disk->subdisk_parent)
 			{
 			if (gkrellmd_check_client_version(mon, 2, 2, 7) && disk->virtual)
-				snprintf(buf, sizeof(buf), "%s virtual %" PRIu64 " %" PRIu64 "\n",
+                buf = g_strdup_printf("%s virtual %" PRIu64 " %" PRIu64 "\n",
 							disk->name, disk->rb, disk->wb);
 			else
-				snprintf(buf, sizeof(buf), "%s %" PRIu64 " %" PRIu64 "\n",
+				buf = g_strdup_printf("%s %" PRIu64 " %" PRIu64 "\n",
 							disk->name, disk->rb, disk->wb);
 			}
 		else if (mon->privat->client->feature_subdisk)
-			snprintf(buf, sizeof(buf), "%s %s %" PRIu64 " %" PRIu64 "\n",
+			buf = g_strdup_printf("%s %s %" PRIu64 " %" PRIu64 "\n",
 						disk->name, disk->subdisk_parent, disk->rb, disk->wb);
 		else
 			continue;
 		gkrellmd_serve_data(mon, buf);
+        g_free(buf);
+        buf = NULL;
 		}
 	}
 
@@ -729,7 +758,7 @@ serve_inet_data(GkrellmdMonitor *mon, gboolean first_serve)
 			   )
 				{
 				cp = inet_ntoa(tcp->remote_addr);
-				sprintf(buf, "+0 %x %s:%x\n",
+				snprintf(buf, sizeof(buf), "+0 %x %s:%x\n",
 							tcp->local_port, cp, tcp->remote_port);
 				}
 #if defined(INET6) && defined(HAVE_GETADDRINFO)
@@ -771,7 +800,7 @@ serve_inet_data(GkrellmdMonitor *mon, gboolean first_serve)
 			if (tcp->family == AF_INET)
 				{
 				cp = inet_ntoa(tcp->remote_addr);
-				sprintf(buf, "-0 %x %s:%x\n",
+				snprintf(buf, sizeof(buf), "-0 %x %s:%x\n",
 							tcp->local_port, cp, tcp->remote_port);
 				}
 #if defined(INET6) && defined(HAVE_GETADDRINFO)
@@ -1031,7 +1060,7 @@ update_net(GkrellmdMonitor *mon, gboolean first_update)
 			if (net_timer->up_event)
 				{
 				snprintf(buf, sizeof(buf), "/var/run/%s.pid", net_timer->name);
-				if (stat(buf, &st) == 0)
+				if (g_stat(buf, &st) == 0)
 					net_timer0 = st.st_mtime;
 				else
 					time(&net_timer0);
@@ -1077,7 +1106,7 @@ serve_net_data(GkrellmdMonitor *mon, gboolean first_serve)
 		net = (NetData *) list->data;
 		if (net->changed || first_serve)
 			{
-			sprintf(buf, "%s %lu %lu\n", net->name, net->rx, net->tx);
+			snprintf(buf, sizeof(buf), "%s %lu %lu\n", net->name, net->rx, net->tx);
 			gkrellmd_serve_data(mon, buf);
 			}
 		}
@@ -1094,7 +1123,7 @@ serve_net_data(GkrellmdMonitor *mon, gboolean first_serve)
 			fake_up_event = (first_serve && net->up);
 			if (net->up_event || net->down_event || fake_up_event)
 				{
-				sprintf(buf, "%s %d\n", net->name,
+				snprintf(buf, sizeof(buf), "%s %d\n", net->name,
 						fake_up_event ? TRUE : net->up_event);
 				gkrellmd_serve_data(mon, buf);
 				}
@@ -1108,7 +1137,7 @@ serve_net_data(GkrellmdMonitor *mon, gboolean first_serve)
 		if (net_timer->timed_changed || first_serve)
 			{
 			gkrellmd_set_serve_name(mon, "net_timer");
-			sprintf(buf, "%s %d\n", net_timer->name, (gint)net_timer->up_time);
+			snprintf(buf, sizeof(buf), "%s %d\n", net_timer->name, (gint)net_timer->up_time);
 			gkrellmd_serve_data(mon, buf);
 			}
 		}
@@ -1498,7 +1527,7 @@ update_fs(GkrellmdMonitor *mon, gboolean first_update)
 #endif
 			}
 		}
-	if (gkrellm_sys_fs_fstab_modified())
+	if (first_update || gkrellm_sys_fs_fstab_modified())
 		refresh_fstab_list();
 
 	gkrellmd_need_serve(mon);
@@ -1521,6 +1550,8 @@ serve_fs_data(GkrellmdMonitor *mon, gboolean first_serve)
 			snprintf(buf, sizeof(buf), "%s %s %s %ld %ld %ld %ld\n",
 					m->directory, m->device, m->type,
 					m->blocks, m->bavail, m->bfree, m->bsize);
+			/*gkrellm_debug(DEBUG_SERVER,
+				"Adding mount-line for %s to serve-data\n", m->directory);*/
 			gkrellmd_serve_data(mon, buf);
 			}
 		}
@@ -1535,6 +1566,8 @@ serve_fs_data(GkrellmdMonitor *mon, gboolean first_serve)
 			snprintf(buf, sizeof(buf), "%s %s %ld %ld %ld %ld\n",
 					m->directory, m->device,
 					m->blocks, m->bavail, m->bfree, m->bsize);
+			/*gkrellm_debug(DEBUG_SERVER,
+				"Updating fs %s in serve-data\n", m->directory);*/
 			gkrellmd_serve_data(mon, buf);
 			}
 		}
@@ -1547,6 +1580,8 @@ serve_fs_data(GkrellmdMonitor *mon, gboolean first_serve)
 			m = (Mount *) list->data;
 			snprintf(buf, sizeof(buf), "%s %s %s\n",
 					m->directory, m->device, m->type);
+			/*gkrellm_debug(DEBUG_SERVER,
+				"Adding fstab-line for %s to serve-data\n", m->directory);*/
 			gkrellmd_serve_data(mon, buf);
 			}
 		}
@@ -1928,7 +1963,7 @@ serve_sensors_data(GkrellmdMonitor *mon, gboolean first_serve)
 			continue;
 		if (sr->changed || first_serve)
 			{
-			sprintf(buf, "%d \"%s\" %d %d %d %.2f\n",
+			snprintf(buf, sizeof(buf), "%d \"%s\" %d %d %d %.2f\n",
 					sr->type, sr->id_name,
 					sr->id, sr->iodev, sr->inter, sr->raw_value);
 			gkrellmd_serve_data(mon, buf);
@@ -1953,12 +1988,12 @@ serve_sensors_setup(GkrellmdMonitor *mon)
 		if (s->group == SENSOR_GROUP_DISK && !sensor_disk_ok)
 			continue;
 		if (sensor_disk_ok)
-			sprintf(buf, "%d \"%s\" %d %d %d %.4f %.4f \"%s\" \"%s\" %d\n",
+			snprintf(buf, sizeof(buf), "%d \"%s\" %d %d %d %.4f %.4f \"%s\" \"%s\" %d\n",
 					s->type, s->id_name,
 					s->id, s->iodev, s->inter,
 					s->factor, s->offset, s->vref, s->default_label, s->group);
 		else
-			sprintf(buf, "%d \"%s\" %d %d %d %.4f %.4f \"%s\" \"%s\"\n",
+			snprintf(buf, sizeof(buf), "%d \"%s\" %d %d %d %.4f %.4f \"%s\" \"%s\"\n",
 					s->type, s->id_name,
 					s->id, s->iodev, s->inter,
 					s->factor, s->offset, s->vref, s->default_label);
@@ -2045,7 +2080,7 @@ send_time(GkrellmdClient *client)
 	gchar		buf[128];
 
 	t = &gkrellmd_current_tm;
-	sprintf(buf, "<time>\n%d %d %d %d %d %d %d %d %d\n",
+	snprintf(buf, sizeof(buf), "<time>\n%d %d %d %d %d %d %d %d %d\n",
 		t->tm_sec, t->tm_min, t->tm_hour,
 		t->tm_mday, t->tm_mon, t->tm_year,
 		t->tm_wday, t->tm_yday, t->tm_isdst);
@@ -2099,7 +2134,7 @@ gkrellmd_load_monitors(void)
 	if (_GK.list_plugins)
 		exit(0);
 	if (_GK.log_plugins)
-		printf("%s\n", plugin_install_log ? plugin_install_log :
+		g_message("%s\n", plugin_install_log ? plugin_install_log :
 				_("No plugins found\n"));
 	for (  ; list; list = list->next)
 		{
@@ -2189,7 +2224,7 @@ gkrellmd_update_monitors(void)
 			send_time(client);
 		else if (GK.second_tick)
 			{
-			sprintf(buf, "<.%d>\n", gkrellmd_current_tm.tm_sec);
+			snprintf(buf, sizeof(buf), "<.%d>\n", gkrellmd_current_tm.tm_sec);
 			gkrellmd_send_to_client(client, buf);
 			}
 
@@ -2229,7 +2264,7 @@ gkrellmd_serve_setup(GkrellmdClient *client)
 	g_free(s);
 
 	lc = localeconv();
-	sprintf(buf, "%c\n", *lc->decimal_point);
+	snprintf(buf, sizeof(buf), "%c\n", *lc->decimal_point);
 	s = g_strconcat("<decimal_point>\n", buf, "\n", NULL);
 	gkrellmd_send_to_client(client, s);
 	g_free(s);
@@ -2257,16 +2292,16 @@ gkrellmd_serve_setup(GkrellmdClient *client)
 	for (list = gkrellmd_monitor_list; list; list = list->next)
 		{
 		mon = (GkrellmdMonitor *) list->data;
-		sprintf(buf, "%s\n", mon->name);
+		snprintf(buf, sizeof(buf), "%s\n", mon->name);
 		gkrellmd_send_to_client(client, buf);
 		}
 
-	sprintf(buf, "%d\n", _GK.io_timeout);
+	snprintf(buf, sizeof(buf), "%d\n", _GK.io_timeout);
 	s = g_strconcat("<io_timeout>\n", buf, "\n", NULL);
 	gkrellmd_send_to_client(client, s);
 	g_free(s);
 
-	sprintf(buf, "%d\n", _GK.reconnect_timeout);
+	snprintf(buf, sizeof(buf), "%d\n", _GK.reconnect_timeout);
 	s = g_strconcat("<reconnect_timeout>\n", buf, "\n", NULL);
 	gkrellmd_send_to_client(client, s);
 	g_free(s);
