@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2007 Bill Wilson
+|  Copyright (C) 1999-2010 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -17,16 +17,45 @@
 |
 |  You should have received a copy of the GNU General Public License
 |  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
 #include "gkrellm-private.h"
 
+#if defined(WIN32)
 
-#define	DEFAULT_CLOCK_FORMAT \
-	"%l:%M <span foreground=\"$A\"><small>%S</small></span>"
-#define	DEFAULT_CAL_FORMAT \
-	"%a <span foreground=\"$A\"><big><big>%e</big></big></span> %b"
+#define DEFAULT_CLOCK_FORMAT "%#I:%M <span foreground=\"$A\"><small>%S</small></span>"
+#define ALT1_CLOCK_FORMAT "%#I:%M <span foreground=\"$A\"><small>%p</small></span>"
+#define ALT2_CLOCK_FORMAT "%#H:%M <span foreground=\"$A\"><small>%S</small></span>"
+#define DEFAULT_CAL_FORMAT "%a <span foreground=\"$A\"><big><big>%#d</big></big></span> %b"
+#define ALT1_CAL_FORMAT "<big>%a %b <span foreground=\"$A\">%#d</span></big>"
+#define ALT2_CAL_FORMAT "%a <span foreground=\"cyan2\"><span font_desc=\"16.5\"><i>%#d</i></span></span> %b"
+
+#else
+
+#define DEFAULT_CLOCK_FORMAT \
+		_("%l:%M <span foreground=\"$A\"><small>%S</small></span>")
+#define ALT1_CLOCK_FORMAT "%l:%M <span foreground=\"$A\"><small>%p</small></span>"
+#define ALT2_CLOCK_FORMAT "%k:%M <span foreground=\"$A\"><small>%S</small></span>"
+
+#define DEFAULT_CAL_FORMAT \
+		_("%a <span foreground=\"$A\"><big><big>%e</big></big></span> %b")
+#define ALT1_CAL_FORMAT  "<big>%a %b <span foreground=\"$A\">%e</span></big>"
+#define ALT2_CAL_FORMAT "%a <span foreground=\"cyan2\"><span font_desc=\"16.5\"><i>%e</i></span></span> %b"
+
+#endif
 
 
 static GkrellmMonitor
@@ -94,9 +123,9 @@ chime_func(gpointer data)
 	ChimeData	*chime = (ChimeData *)data;
 	gint		counter;
 
-	if (strlen(chime->command)) 
+	if (strlen(chime->command))
 		{
-		if (chime->count > 12) 
+		if (chime->count > 12)
 			chime->count -= 12;
 
 		for (counter = 0; counter < chime -> count; counter ++)
@@ -118,6 +147,7 @@ get_color_name(GdkColor *color, gchar **color_string)
 			(color->green >> 8) & 0xff,
 			(color->blue >> 8) & 0xff );
 	gkrellm_dup_string(color_string, cstring);
+	g_free(cstring);
 	}
 
 static void
@@ -166,6 +196,7 @@ static gchar *
 strftime_format(gchar *format, gchar *alt_color)
 	{
 	struct tm	*t;
+	gchar		*locale_format = NULL;
 	gchar		buf1[512], buf2[512];
 
 	if (_GK.client_mode)
@@ -173,12 +204,14 @@ strftime_format(gchar *format, gchar *alt_color)
 	else
 		t = &gkrellm_current_tm;
 
-	strftime(buf1, sizeof(buf1), format, t);
+	locale_format = g_locale_from_utf8 (format, -1, NULL, NULL, NULL);
+	strftime(buf1, sizeof(buf1), locale_format, t);
+	g_free (locale_format);
 	format_alt_color(buf1, buf2, sizeof(buf2), alt_color);
 
-// printf("%s\n", buf2);
+// g_debug("%s\n", buf2);
 
-	return g_strdup(buf2);
+	return g_locale_to_utf8 (buf2, -1, NULL, NULL, NULL);
 	}
 
 static gint
@@ -237,6 +270,7 @@ create_calendar_panel(GtkWidget *vbox, gint first_create)
 	cal_string = strftime_format(cal_format, cal_alt_color_string);
 	d_cal = gkrellm_create_decal_text_markup(pcal, cal_string,
 				gkrellm_meter_textstyle(cal_style_id), style, -1, -1, -1);
+	g_free(cal_string);
 
 	gkrellm_panel_configure(pcal, NULL, style);
 	gkrellm_panel_create(vbox, mon_cal, pcal);
@@ -297,8 +331,9 @@ draw_cal(void)
 
 	gkrellm_draw_decal_markup(pcal, d_cal, cal_string);
 	gkrellm_decal_text_set_offset(d_cal, (d_cal->w - w) / 2, 0);
-	
+
 	gkrellm_draw_panel_layers(pcal);
+	g_free(cal_string);
 	}
 
 static void
@@ -341,6 +376,7 @@ create_clock_panel(GtkWidget *vbox, gint first_create)
 	clock_string = strftime_format(clock_format, clock_alt_color_string);
 	d_clock = gkrellm_create_decal_text_markup(pclock, clock_string,
 				gkrellm_meter_textstyle(clock_style_id), style, -1, -1, -1);
+	g_free(clock_string);
 
 	gkrellm_panel_configure(pclock, NULL, style);
 	gkrellm_panel_create(vbox, mon_clock, pclock);
@@ -396,6 +432,7 @@ draw_clock(gboolean check_size)
 		}
 	gkrellm_draw_decal_markup(pclock, d_clock, clock_string);
 	gkrellm_draw_panel_layers(pclock);
+	g_free(clock_string);
 	}
 
 
@@ -437,7 +474,7 @@ update_clock(void)
 		draw_cal();
 		if (ptm->tm_hour != hour_prev && hour_prev != -1)
 			{
-			if (!chime_block && hour_chime_command && *hour_chime_command) 
+			if (!chime_block && hour_chime_command && *hour_chime_command)
 				{
 				chime = g_new0(ChimeData, 1);
 				chime -> command = strdup(hour_chime_command);
@@ -449,7 +486,7 @@ update_clock(void)
 			{
 			if (   !chime_block && (ptm->tm_min % 15) == 0
 				&& quarter_chime_command && *quarter_chime_command
-			   ) 
+			   )
 				{
 				chime = g_new0(ChimeData, 1);
 				chime -> command = strdup(quarter_chime_command);
@@ -538,8 +575,8 @@ static GtkWidget	*cal_launch_entry,
 					*cal_enable_button,
 					*loop_chime_button;
 
-static GtkWidget	*cal_format_combo,
-					*clock_format_combo;
+static GtkWidget	*cal_format_combo_box,
+					*clock_format_combo_box;
 
 
 static void
@@ -592,8 +629,10 @@ static void
 cal_format_cb(GtkWidget *widget, gpointer data)
 	{
 	gchar	*s, *check;
+	GtkWidget *entry;
 
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(cal_format_combo)->entry));
+	entry = gtk_bin_get_child(GTK_BIN(cal_format_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
 
 	check = strftime_format(s, cal_alt_color_string);
 
@@ -617,8 +656,10 @@ static void
 clock_format_cb(GtkWidget *widget, gpointer data)
 	{
 	gchar	*s, *check;
+	GtkWidget *entry;
 
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(clock_format_combo)->entry));
+	entry = gtk_bin_get_child(GTK_BIN(clock_format_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
 
 	check = strftime_format(s, clock_alt_color_string);
 
@@ -662,7 +703,6 @@ create_clock_tab(GtkWidget *tab_vbox)
 	{
 	GtkWidget	*tabs;
 	GtkWidget	*table, *vbox, *vbox1, *hbox, *label, *text;
-	GList		*list;
 	gint		i;
 
 	tabs = gtk_notebook_new();
@@ -682,22 +722,14 @@ create_clock_tab(GtkWidget *tab_vbox)
 	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 6);
 	label = gtk_label_new(_("Display format string:"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
-	cal_format_combo = gtk_combo_new();
-	gtk_box_pack_start(GTK_BOX(vbox1), cal_format_combo, TRUE, TRUE, 0);
-	list = NULL;
-	list = g_list_append(list, cal_format);
-	list = g_list_append(list, DEFAULT_CAL_FORMAT);
-	list = g_list_append(list,
-"<big>%a %b <span foreground=\"$A\">%e</span></big>");
-	list = g_list_append(list,
-"%a <span foreground=\"cyan2\"><span font_desc=\"16.5\"><i>%e</i></big></span></span> %b");
-
-	gtk_combo_set_popdown_strings(GTK_COMBO(cal_format_combo), list);
-	gtk_combo_set_case_sensitive(GTK_COMBO(cal_format_combo), TRUE);
-	g_list_free(list);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(cal_format_combo)->entry),
-			cal_format);
-	g_signal_connect(G_OBJECT(GTK_COMBO(cal_format_combo)->entry), "changed",
+	cal_format_combo_box = gtk_combo_box_entry_new_text();
+	gtk_box_pack_start(GTK_BOX(vbox1), cal_format_combo_box, TRUE, TRUE, 0);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(cal_format_combo_box), cal_format);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(cal_format_combo_box), DEFAULT_CAL_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(cal_format_combo_box), ALT1_CAL_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(cal_format_combo_box), ALT2_CAL_FORMAT);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(cal_format_combo_box), 0);
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(cal_format_combo_box)), "changed",
 			G_CALLBACK(cal_format_cb), NULL);
 
 	vbox1 = gkrellm_gtk_category_vbox(vbox, _("Clock"), 4, 0, TRUE);
@@ -709,22 +741,14 @@ create_clock_tab(GtkWidget *tab_vbox)
 	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 6);
 	label = gtk_label_new(_("Display format string:"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
-	clock_format_combo = gtk_combo_new();
-	gtk_box_pack_start(GTK_BOX(vbox1), clock_format_combo, TRUE, TRUE, 0);
-	list = NULL;
-	list = g_list_append(list, clock_format);
-	list = g_list_append(list, DEFAULT_CLOCK_FORMAT);
-	list = g_list_append(list,
-		"%l:%M <span foreground=\"$A\"><small>%p</small></span>");
-	list = g_list_append(list,
-		"%k:%M <span foreground=\"$A\"><small>%S</small></span>");
-
-	gtk_combo_set_popdown_strings(GTK_COMBO(clock_format_combo), list);
-	gtk_combo_set_case_sensitive(GTK_COMBO(clock_format_combo), TRUE);
-	g_list_free(list);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(clock_format_combo)->entry),
-			clock_format);
-	g_signal_connect(G_OBJECT(GTK_COMBO(clock_format_combo)->entry), "changed",
+	clock_format_combo_box = gtk_combo_box_entry_new_text();
+	gtk_box_pack_start(GTK_BOX(vbox1), clock_format_combo_box, TRUE, TRUE, 0);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(clock_format_combo_box), clock_format);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(clock_format_combo_box), DEFAULT_CLOCK_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(clock_format_combo_box), ALT1_CLOCK_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(clock_format_combo_box), ALT2_CLOCK_FORMAT);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(clock_format_combo_box), 0);
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(clock_format_combo_box)), "changed",
 			G_CALLBACK(clock_format_cb), NULL);
 
 /* -- Setup tab */
@@ -768,13 +792,11 @@ create_clock_tab(GtkWidget *tab_vbox)
 				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	for (i = 0; i < sizeof(clock_info_text)/sizeof(gchar *); ++i)
 		gkrellm_gtk_text_view_append(text, _(clock_info_text[i]));
-
-
 	g_signal_connect(G_OBJECT(cal_launch_entry), "changed",
 			G_CALLBACK(cb_launch_entry), GINT_TO_POINTER(1));
 	g_signal_connect(G_OBJECT(cal_tooltip_entry), "changed",
 			G_CALLBACK(cb_launch_entry), GINT_TO_POINTER(1));
-	
+
 	gkrellm_gtk_config_launcher(table, 1,  &clock_launch_entry,
 				&clock_tooltip_entry, _("Clock"), &clock_launch);
 	g_signal_connect(G_OBJECT(clock_launch_entry), "changed",
