@@ -1,22 +1,34 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2010 Bill Wilson
 |
-|  Author:  Bill Wilson    bill@gkrellm.net
+|  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -910,10 +922,11 @@ create_cpu(GtkWidget *vbox, gint first_create)
 /* ------------------------------------------------------------------ */
 #define	CPU_CONFIG_KEYWORD	"cpu"
 
-static GtkWidget	*text_format_combo;
+static GtkWidget	*text_format_combo_box;
 static GtkWidget	*smp_button[3];
+#if !defined(WIN32)
 static GtkWidget	*alert_config_nice_button;
-
+#endif
 
 static void
 cb_alert_trigger(GkrellmAlert *alert, CpuMon *cpu)
@@ -1169,8 +1182,10 @@ cb_text_format(GtkWidget *widget, gpointer data)
 	GList	*list;
 	CpuMon	*cpu;
 	gchar	*s;
+	GtkWidget *entry;
 
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(text_format_combo)->entry));
+	entry = gtk_bin_get_child(GTK_BIN(text_format_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
 	gkrellm_locale_dup_string(&text_format, s, &text_format_locale);
 	for (list = cpu_mon_list; list; list = list->next)
 		{
@@ -1274,7 +1289,7 @@ create_cpu_tab(GtkWidget *tab_vbox)
 	{
 	GtkWidget	*tabs;
 	GtkWidget	*button;
-	GtkWidget	*hbox, *hbox1, *vbox, *vbox1;
+	GtkWidget	*hbox, *vbox, *vbox1;
 	GtkWidget	*text;
 	GtkWidget	*table;
 	GSList		*group;
@@ -1312,8 +1327,9 @@ create_cpu_tab(GtkWidget *tab_vbox)
 				cb_config_tracking, NULL,
 		_("Apply any CPU chart config height change to all CPU charts"));
 
-		vbox1 = gkrellm_gtk_framed_vbox(vbox, _("SMP Charts Select"),
-				4, FALSE, 0, 2);
+		vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("SMP Charts Select"),
+				4, 0, TRUE);
 
 		button = gtk_radio_button_new_with_label(NULL, _("Real CPUs."));
 		gtk_box_pack_start(GTK_BOX(vbox1), button, TRUE, TRUE, 0);
@@ -1340,39 +1356,38 @@ create_cpu_tab(GtkWidget *tab_vbox)
 
 /* -- Setup tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
+
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Format String for Chart Labels"),
+				4, 0, TRUE);
+
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 5);
-	vbox1 = gkrellm_gtk_framed_vbox(hbox, _("Format String for Chart Labels"),
-			 4, FALSE, 0, 2);
-	hbox1 = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, TRUE, 5);
-	text_format_combo = gtk_combo_new();
-	gtk_widget_set_size_request (GTK_WIDGET(text_format_combo), 300, -1);
-	gtk_box_pack_start(GTK_BOX(hbox1), text_format_combo, FALSE, TRUE, 0);
-	list = NULL;
-	list = g_list_append(list, text_format);
-	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
-	list = g_list_append(list, _("\\fu \\.$u\\n\\fs \\.$s"));
-	list = g_list_append(list, _("\\ww\\D2\\f\\au\\.$u\\D1\\f\\as\\.$s"));
-	list = g_list_append(list, _("\\ww\\D3\\f\\au\\.$u\\D0\\f\\as\\.$s"));
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
+
+	text_format_combo_box = gtk_combo_box_entry_new_text();
+	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo_box, TRUE, TRUE, 0);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		text_format);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		DEFAULT_TEXT_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		_("\\fu \\.$u\\n\\fs \\.$s"));
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		_("\\ww\\D2\\f\\au\\.$u\\D1\\f\\as\\.$s"));
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		_("\\ww\\D3\\f\\au\\.$u\\D0\\f\\as\\.$s"));
 	if (!nice_time_unsupported)
-		list = g_list_append(list,
+		{
+		gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
 			"\\ww\\C\\f$L\\D5\\f\\an\\.$n\\D2\\f\\au\\.$u\\D1\\f\\as\\.$s");
-	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
-	gtk_combo_set_case_sensitive(GTK_COMBO(text_format_combo), TRUE);
-	g_list_free(list);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
-			text_format);
-	g_signal_connect(G_OBJECT(GTK_COMBO(text_format_combo)->entry), "changed",
+		}
+	gtk_combo_box_set_active(GTK_COMBO_BOX(text_format_combo_box), 0);
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(text_format_combo_box)), "changed",
 			G_CALLBACK(cb_text_format), NULL);
 
-	vbox1 = gtk_vbox_new(FALSE, 0);
-	gtk_box_pack_end(GTK_BOX(hbox), vbox1, FALSE, TRUE, 4);
-	gkrellm_gtk_alert_button(vbox1, NULL, FALSE, FALSE, 4, FALSE,
-			cb_set_alert, NULL);
-
-	vbox1 = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),
-				4, TRUE, 0,2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Launch Commands"),
+				4, 0, TRUE);
 	vbox1 = gkrellm_gtk_scrolled_vbox(vbox1, NULL,
 				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	table = gkrellm_gtk_launcher_table_new(vbox1, n_cpus);
@@ -1388,6 +1403,11 @@ create_cpu_tab(GtkWidget *tab_vbox)
 				G_CALLBACK(cb_launch_entry), cpu);
 		}
 
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gkrellm_gtk_alert_button(hbox, NULL, FALSE, FALSE, 4, TRUE,
+			cb_set_alert, NULL);
+
 /* --Info tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Info"));
 	text = gkrellm_gtk_scrolled_text_view(vbox, NULL,
