@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2007 Bill Wilson
+|  Copyright (C) 1999-2010 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -17,6 +17,18 @@
 |
 |  You should have received a copy of the GNU General Public License
 |  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include <string.h>
@@ -88,14 +100,14 @@ gdk_string_extents(void *font_desc, const char *string,
 int
 setuid(uid_t u)
 	{
-//	printf("setuid() intercepted\n");
+//	g_debug("setuid() intercepted\n");
 	return 0;
 	}
 
 int
 setreuid(uid_t r, uid_t e)
 	{
-//	printf("setreuid() intercepted\n");
+//	g_debug("setreuid() intercepted\n");
 	return 0;
 	}
 
