@@ -1,22 +1,34 @@
 /* GKrellM
-|  Copyright (C) 1999-2005 Bill Wilson
+|  Copyright (C) 1999-2010 Bill Wilson
 |
-|  Author:  Bill Wilson    bill@gkrellm.net
+|  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -24,7 +36,10 @@
 #include "gkrellm-sysdeps.h"
 
 
-#define	DEFAULT_DATA_FORMAT	_("$t - $f free")
+#define	DEFAULT_DATA_FORMAT	(_("$t - $f free"))
+#define	ALT1_DATA_FORMAT (_("$t - $u used"))
+#define	ALT2_DATA_FORMAT (_("$t - $U"))
+
 
   /* Values for force_fs_check	*/
 #define	FORCE_REDRAW	1
@@ -51,7 +66,8 @@ typedef struct
 				*drawer_button;
 	GkrellmDecal *mount_decal,
 				*eject_decal,
-				*text_decal;
+				*label_decal,
+				*data_decal;
 	GkrellmKrell *krell;
 	gchar		*label,			/* Actual utf8 label */
 				*label_shadow;	/* Shadow label for gdk_draw functions */
@@ -133,7 +149,7 @@ static gint		x_eject_button_open,
 				x_eject_button_closed;
 
 static gint		x_scroll;
-static gint		text_decal_width;
+static gint		data_decal_width;
 static gint		cdrom_auto_eject;
 static gint		binary_units;
 static gboolean	mounting_supported = TRUE,
@@ -154,8 +170,11 @@ static gchar *remote_fs_types[]	=
 static gboolean
 setup_fs_interface(void)
     {
+#ifdef WIN32
+	uid = 0; /* nothing comparable available on windows */
+#else
 	uid = getuid();	/* only real root is allowed to mount/umount always */
-
+#endif
 	if (!get_fsusage && !_GK.client_mode && gkrellm_sys_fs_init())
 		{
 		get_fsusage = gkrellm_sys_fs_get_fsusage;
@@ -363,9 +382,12 @@ format_fs_data(FSmon *fs, gchar *src_string, gchar *buf, gint size)
 			? (sizeof(fs_binary_abbrev) / sizeof(GkrellmSizeAbbrev))
 			: (sizeof(fs_decimal_abbrev) / sizeof(GkrellmSizeAbbrev));
 
-	gkrellm_format_size_abbrev(tbuf, sizeof(tbuf), (gfloat) b * bsize, tbl, tbl_size);
-	gkrellm_format_size_abbrev(ubuf, sizeof(ubuf), (gfloat) u * bsize, tbl, tbl_size);
-	gkrellm_format_size_abbrev(abuf, sizeof(abuf), (gfloat) a * bsize, tbl, tbl_size);
+	gkrellm_format_size_abbrev(tbuf, sizeof(tbuf), (gfloat) b * bsize,
+				tbl, tbl_size);
+	gkrellm_format_size_abbrev(ubuf, sizeof(ubuf), (gfloat) u * bsize,
+				tbl, tbl_size);
+	gkrellm_format_size_abbrev(abuf, sizeof(abuf), (gfloat) a * bsize,
+				tbl, tbl_size);
 
 	for (s = src_string; *s != '\0' && size > 0; ++s)
 		{
@@ -432,21 +454,26 @@ format_fs_data(FSmon *fs, gchar *src_string, gchar *buf, gint size)
   /* Draw the fs label or toggle the fs total blocks and blocks avail.
   */
 static gint
-draw_fs_text_decal(FSmon *fs, gint value)
+fs_draw_decal_text(FSmon *fs, gint value)
 	{
 	GkrellmDecal		*d;
 	GkrellmTextstyle	ts_save;
 	gchar				buf[128];
 	gint				x_off, w;
 
-	d = fs->text_decal;
 	if (value == 0)
 		{
+		gkrellm_make_decal_invisible(fs->panel, fs->data_decal);
+		d = fs->label_decal;
+		gkrellm_make_decal_visible(fs->panel, d);
 		gkrellm_decal_text_set_offset(d, 0, 0);
-		gkrellm_draw_decal_text(fs->panel, d, fs->label_shadow, 0);
+		gkrellm_draw_decal_markup(fs->panel, d, fs->label_shadow);
 		}
 	else if (!fs->busy)
 		{
+		gkrellm_make_decal_invisible(fs->panel, fs->label_decal);
+		d = fs->data_decal;
+		gkrellm_make_decal_visible(fs->panel, d);
 		ts_save = d->text_style;
 		d->text_style = *gkrellm_meter_alt_textstyle(style_id);
 
@@ -571,7 +598,9 @@ pipe_command(FSmon *fs, gchar *command)
 	snprintf(buf, sizeof(buf), "%s 2>&1", command);
 	if ((fs->launch_mount.pipe = popen(buf, "r")) == NULL)
 		return;
+#ifndef WIN32
 	fcntl(fileno(fs->launch_mount.pipe), F_SETFL, O_NONBLOCK);
+#endif
 	}
 
 static void
@@ -584,11 +613,11 @@ mount_command(FSmon *fs)
 	if (fs->is_mounted)
 		{
 		if (fs->fstab_mounting)
-			snprintf(cmd, sizeof(cmd), "umount %s", fs->mount.directory);
+			snprintf(cmd, sizeof(cmd), "umount '%s'", fs->mount.directory);
 		else
 			snprintf(cmd, sizeof(cmd), "%s", fs->launch_umount.command);
 		fs->label_is_data = FALSE;
-		draw_fs_text_decal(fs, 0);
+		fs_draw_decal_text(fs, 0);
 		pipe_command(fs, cmd);
 		if (cdrom_auto_eject)
 			fs->eject_pending = GK.timer_ticks + 5;	/* at least 1/2 sec delay*/
@@ -598,7 +627,7 @@ mount_command(FSmon *fs)
 		if (fs->ejectable)
 			close_tray(fs);
 		if (fs->fstab_mounting)
-			snprintf(cmd, sizeof(cmd), "mount %s", fs->mount.directory);
+			snprintf(cmd, sizeof(cmd), "mount '%s'", fs->mount.directory);
 		else
 			snprintf(cmd, sizeof(cmd), "%s", fs->launch_mount.command);
 		fs->blocks = fs->bfree = fs->bavail = fs->bsize = 0;
@@ -681,7 +710,7 @@ animate_eject_button(FSmon *fs, gboolean force_close)
 	}
 
 static void
-update_fs(void)
+fs_update(void)
 	{
 	FSmon			*fs;
 	GkrellmPanel	*p;
@@ -701,7 +730,7 @@ update_fs(void)
 		fs = (FSmon *) list->data;
 		if (fs->label_is_data && !fs_in_motion)
 			{
-			w = draw_fs_text_decal(fs, 1);
+			w = fs_draw_decal_text(fs, 1);
 			if (w > w_scroll)
 				w_scroll = w;
 			gkrellm_draw_panel_layers(fs->panel);
@@ -709,9 +738,9 @@ update_fs(void)
 		}
 	if (!fs_in_motion)
 		{
-		if (w_scroll > text_decal_width)
+		if (w_scroll > data_decal_width)
 			x_scroll = (x_scroll + ((gkrellm_update_HZ() < 7) ? 2 : 1))
-					% (w_scroll - text_decal_width / 3);
+					% (w_scroll - data_decal_width / 3);
 		else
 			x_scroll = 0;
 		}
@@ -725,9 +754,7 @@ update_fs(void)
 
 	if (!force_fs_check && (!GK.second_tick || (!fs_check && !nfs_check)))
 		return;
-#if 0
-printf("fs update %d nfs %d force %d\n", fs_check, nfs_check, force_fs_check);
-#endif
+//g_debug("fs update %d nfs %d force %d\n", fs_check, nfs_check, force_fs_check);
 
 	refresh_mounts_list();
 
@@ -824,7 +851,7 @@ printf("fs update %d nfs %d force %d\n", fs_check, nfs_check, force_fs_check);
 			}
 		gkrellm_set_button_sensitive(fs->md_button, mounting_enabled);
 		if (!fs->label_is_data && fs != fs_in_motion)
-			draw_fs_text_decal(fs, 0);
+			fs_draw_decal_text(fs, 0);
 
 		if (_GK.client_mode)
 			{
@@ -949,7 +976,7 @@ cb_panel_release(GtkWidget *widget, GdkEventButton *ev, FSmon *fs)
 			if (fs_in_motion->label_is_data)
 				gkrellm_config_modified();
 			fs_in_motion->label_is_data = FALSE;
-			draw_fs_text_decal(fs_in_motion, 0);
+			fs_draw_decal_text(fs_in_motion, 0);
 			gkrellm_show_button(fs_in_motion->drawer_button);
 			gkrellm_draw_panel_layers(fs_in_motion->panel);
 			}
@@ -991,7 +1018,7 @@ cb_panel_press(GtkWidget *widget, GdkEventButton *ev, FSmon *fs)
 		gkrellm_config_modified();
 		}
 	x_fs_motion = ev->x;
-	draw_fs_text_decal(fs, 1);
+	fs_draw_decal_text(fs, 1);
 	gkrellm_draw_panel_layers(fs->panel);
 	fs_in_motion = fs;
 	x_moved = 0;
@@ -1021,7 +1048,7 @@ cb_panel_motion(GtkWidget *widget, GdkEventButton *ev)
 		}
 	font_desc = gkrellm_meter_alt_textstyle(style_id)->font;
 
-	d = fs_in_motion->text_decal;
+	d = fs_in_motion->data_decal;
 	format_fs_data(fs_in_motion, data_format_locale, buf, sizeof(buf));
 	gkrellm_decal_scroll_text_get_size(d, &w, NULL);
 	if (w > d->w)
@@ -1039,7 +1066,7 @@ cb_panel_motion(GtkWidget *widget, GdkEventButton *ev)
 			fs = (FSmon *) list->data;
 			if (fs->label_is_data)
 				{
-				draw_fs_text_decal(fs, 1);
+				fs_draw_decal_text(fs, 1);
 				gkrellm_draw_panel_layers(fs->panel);
 				}
 			}
@@ -1074,12 +1101,15 @@ cb_fs_close_tray(GkrellmDecalbutton *button, FSmon *fs)
 	}
 
 static void
-create_fs_monitor(GtkWidget *vbox, FSmon *fs, gint index, gint first_create)
+fs_monitor_create(GtkWidget *vbox, FSmon *fs, gint index, gint first_create)
 	{
-	GkrellmStyle	*style;
-	GkrellmMargin	*m;
-	GkrellmPanel	*p;
-	gint			h, label_x_position, label_y_off;
+	GkrellmStyle		*style;
+	GkrellmTextstyle	*ts;
+	GkrellmMargin		*m;
+	GkrellmPanel		*p;
+	gchar				buf[256];
+	gint				h, label_x_position, label_y_off;
+	gint				h_data, h_label;
 
 	if (first_create)
 		fs->panel = gkrellm_panel_new0();
@@ -1089,13 +1119,30 @@ create_fs_monitor(GtkWidget *vbox, FSmon *fs, gint index, gint first_create)
 	fs->krell_factor = 1;
 
 	style = gkrellm_meter_style(style_id);
+	ts = gkrellm_meter_textstyle(style_id);
+	m = gkrellm_get_style_margins(style);
+
 	gkrellm_panel_label_get_position(style, &label_x_position, &label_y_off);
 
-	m = gkrellm_get_style_margins(style);
-	fs->text_decal = gkrellm_create_decal_text(p, "Afp0",
-			gkrellm_meter_textstyle(style_id), style, -1,
+	format_fs_data(fs, data_format_locale, buf, sizeof(buf));
+	fs->data_decal = gkrellm_create_decal_text_markup(p, buf,
+			ts, style, -1,
 			(label_y_off > 0) ? label_y_off : -1,
 			-1);
+	gkrellm_decal_get_size(fs->data_decal, NULL, &h_data);
+
+	fs->label_decal = gkrellm_create_decal_text_markup(p, fs->label_shadow,
+			ts, style, -1,
+			(label_y_off > 0) ? label_y_off : -1,
+			-1);
+	gkrellm_decal_get_size(fs->label_decal, NULL, &h_label);
+
+	if (h_data > h_label)
+		gkrellm_move_decal(p, fs->label_decal, fs->label_decal->x,
+				fs->label_decal->y + (h_data - h_label + 1) / 2);
+	else if (h_data < h_label)
+		gkrellm_move_decal(p, fs->data_decal, fs->data_decal->x,
+				fs->data_decal->y + (h_label - h_data + 1) / 2);
 
 	fs->mount_decal = gkrellm_create_decal_pixmap(p,
 			gkrellm_decal_misc_pixmap(), gkrellm_decal_misc_mask(),
@@ -1124,7 +1171,7 @@ create_fs_monitor(GtkWidget *vbox, FSmon *fs, gint index, gint first_create)
 
 	/* Usable width to determine various scrolling parameters.
 	*/
-	text_decal_width = fs->mount_decal->x - fs->text_decal->x;
+	data_decal_width = fs->mount_decal->x - fs->data_decal->x;
 
 	fs->krell = gkrellm_create_krell(p,
 						gkrellm_krell_meter_piximage(style_id), style);
@@ -1178,7 +1225,7 @@ create_fs_monitor(GtkWidget *vbox, FSmon *fs, gint index, gint first_create)
 			gkrellm_panel_hide(fs->panel);
 		}
 
-	draw_fs_text_decal(fs, 0);
+	fs_draw_decal_text(fs, 0);
 	force_fs_check = FORCE_UPDATE;
 
 	if (fs->launch_mount.command == NULL)
@@ -1209,7 +1256,7 @@ destroy_fs_monitor(FSmon *fs)
 	}
 
 static void
-create_fs(GtkWidget *vbox, gint first_create)
+fs_create(GtkWidget *vbox, gint first_create)
 	{
 	GList	*list;
 	FSmon	*fs;
@@ -1230,7 +1277,7 @@ create_fs(GtkWidget *vbox, gint first_create)
 	for (i = 0, list = fs_mon_list; list; ++i, list = list->next)
 		{
 		fs = (FSmon *)list->data;
-		create_fs_monitor(fs->secondary ? fs_secondary_vbox : fs_main_vbox,fs,
+		fs_monitor_create(fs->secondary ? fs_secondary_vbox : fs_main_vbox,fs,
 					i, first_create);
 		}
 	if (g_list_length(fs_mon_list) == 0)
@@ -1260,18 +1307,19 @@ create_alert(FSmon *fs)
 	}
 
 static void
-save_fs_config(FILE *f)
+fs_config_save(FILE *f)
 	{
 	GList	*list;
 	FSmon	*fs;
-	gchar	quoted_label[64];
+	gchar	quoted_label[64], quoted_dir[512];
 
 	for (list = fs_mon_list; list; list = list->next)
 		{
 		fs = (FSmon *) list->data;
 		snprintf(quoted_label, sizeof(quoted_label), "\"%s\"", fs->label);
+		snprintf(quoted_dir, sizeof(quoted_dir), "\"%s\"",fs->mount.directory);
 		fprintf(f, "%s %s %s %d %d %d %d %d\n", FS_CONFIG_KEYWORD,
-				quoted_label, fs->mount.directory,
+				quoted_label, quoted_dir,
 				fs->fstab_mounting, fs->secondary,
 				fs->show_if_mounted, fs->label_is_data, fs->ejectable);
 		if (*(fs->launch_mount.command))
@@ -1303,7 +1351,7 @@ static gboolean
 fstab_user_permission(Mount *m)
 	{
 	struct stat my_stat;
-	
+
 	stat(m->device, &my_stat);
 	if (   strstr(m->options, "user")
 		|| (strstr(m->options, "owner") && my_stat.st_uid == uid)
@@ -1326,7 +1374,7 @@ fix_fstab_mountable_changed(FSmon *fs)
 		{
 		fs->fstab_mounting = FALSE;
 		return TRUE;
-		}		
+		}
 	return FALSE;
 	}
 
@@ -1348,12 +1396,12 @@ lookup_fs(gchar *name)
 	}
 
 static void
-load_fs_config(gchar *arg)
+fs_config_load(gchar *arg)
 	{
 	static FSmon	*fs_prev;
 	FSmon			*fs;
-	gchar			*label;
-	gchar			config[32], item[CFG_BUFSIZE], dir[CFG_BUFSIZE];
+	gchar			*cut_label, *cut_dir;
+	gchar			config[32], item[CFG_BUFSIZE];
 	gchar			name[64], item1[CFG_BUFSIZE];
 	gint 			n;
 
@@ -1400,13 +1448,14 @@ load_fs_config(gchar *arg)
 		}
 	else
 		{
-		if ((label = gkrellm_cut_quoted_string(arg, &arg)) != NULL)
+		if (   (cut_label = gkrellm_cut_quoted_string(arg, &arg)) != NULL
+			&& (cut_dir = gkrellm_cut_quoted_string(arg, &arg)) != NULL
+	       )
 			{
 			fs = g_new0(FSmon, 1);
-			gkrellm_locale_dup_string(&fs->label, label, &fs->label_shadow);
-			dir[0] = '\0';
+			gkrellm_locale_dup_string(&fs->label, cut_label,&fs->label_shadow);
 
-			sscanf(arg, "%s %d %d %d %d %d",  dir, &fs->fstab_mounting,
+			sscanf(arg, "%d %d %d %d %d", &fs->fstab_mounting,
 					&fs->secondary, &fs->show_if_mounted,
 					&fs->label_is_data, &fs->ejectable);
 			if (fs->fstab_mounting > 1)		/* pre 2.0.0 config fix */
@@ -1417,7 +1466,7 @@ load_fs_config(gchar *arg)
 				fs->fstab_mounting = fs->show_if_mounted = FALSE;
 			if (fs->secondary)
 				have_secondary_panels = TRUE;
-			fs->mount.directory = g_strdup(dir);
+			fs->mount.directory = g_strdup(cut_dir);
 			fs->restore_label = fs->label_is_data;
 
 			fix_fstab_mountable_changed(fs);
@@ -1458,7 +1507,7 @@ static GtkTreeSelection	*selection;
 
 static GtkWidget
 				*label_entry,
-				*dir_combo,
+				*dir_combo_box,
 				*mount_entry,
 				*umount_entry,
 				*mounting_button,
@@ -1471,7 +1520,7 @@ static GtkWidget
 
 static GtkWidget	*alert_button;
 
-static GtkWidget	*data_format_combo;
+static GtkWidget	*data_format_combo_box;
 
 static gboolean	(*original_row_drop_possible)();
 
@@ -1629,14 +1678,17 @@ cb_alert_config(GkrellmAlert *ap, FSmon *fs)
   |  fstab entries and accordingly set sensitivity of the mounting_button.
   */
 static void
-cb_combo_changed(GtkWidget *widget)
+cb_combo_changed(GtkComboBox *widget, gpointer user_data)
 	{
 	Mount	*m;
 	gchar	*s;
+	GtkWidget *entry;
 
 	if (!mounting_supported || _GK.client_mode)
 		return;
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(dir_combo)->entry));
+
+	entry = gtk_bin_get_child(GTK_BIN(dir_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
 	m = in_fstab_list(s);
 	if (m && (fstab_user_permission(m) || uid == 0))
 		{
@@ -1728,7 +1780,7 @@ static void
 reset_entries(gboolean level0)
 	{
 	gtk_entry_set_text(GTK_ENTRY(label_entry), "");
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dir_combo)->entry), "");
+	gtk_combo_box_set_active(GTK_COMBO_BOX(dir_combo_box), -1);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(secondary_button), level0);
 	if (mounting_button)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mounting_button),FALSE);
@@ -1795,7 +1847,7 @@ cb_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
 	indices = gtk_tree_path_get_indices(path);
 	secondary = indices[0];
 	depth = gtk_tree_path_get_depth(path);
-// printf("selection: indices=[%d,%d]:%d, path=%s\n",
+// g_debug("selection: indices=[%d,%d]:%d, path=%s\n",
 //			indices[0], indices[1], gtk_tree_path_get_depth(path),
 //			gtk_tree_path_to_string(path));
 	change_row_reference(model, path);
@@ -1820,8 +1872,8 @@ cb_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(secondary_button),
 					secondary);
 	gtk_entry_set_text(GTK_ENTRY(label_entry), fs->label);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dir_combo)->entry),
-				fs->mount.directory);
+	gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(dir_combo_box))),
+			fs->mount.directory);
 	if (show_button)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_button),
 					fs->show_if_mounted);
@@ -1891,7 +1943,7 @@ N_("At least one primary fs monitor must exist to click on in order to show\n"
 			fs->secondary = FALSE;
 			fs->show_if_mounted = FALSE;
 			fs_mon_list = g_list_append(fs_mon_list, fs);
-			create_fs_monitor(fs_main_vbox, fs, n_fs_monitors, TRUE);
+			fs_monitor_create(fs_main_vbox, fs, n_fs_monitors, TRUE);
 			gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
 						FSMON_COLUMN, fs, -1);
 			gkrellm_alert_trigger_connect(fs->alert, cb_alert_trigger, fs);
@@ -1907,7 +1959,7 @@ N_("At least one primary fs monitor must exist to click on in order to show\n"
 			fs = fs_new_from_model(model, &iter);
 			fs->secondary = TRUE;
 			fs_mon_list = g_list_append(fs_mon_list, fs);
-			create_fs_monitor(fs_secondary_vbox, fs, n_fs_monitors, TRUE);
+			fs_monitor_create(fs_secondary_vbox, fs, n_fs_monitors, TRUE);
 			gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
 						FSMON_COLUMN, fs, -1);
 			gkrellm_alert_trigger_connect(fs->alert, cb_alert_trigger, fs);
@@ -1935,6 +1987,7 @@ add_cb(GtkWidget *widget)
 	gchar			*label;
 	gint			secondary, *indices;
 	gboolean		a, b, err = FALSE;
+	GtkWidget		*entry;
 
 	fs = g_new0(FSmon, 1);
 	fs->launch_mount.command = g_strdup("");
@@ -1943,8 +1996,10 @@ add_cb(GtkWidget *widget)
 
 	label = gkrellm_gtk_entry_get_text(&label_entry);
 	gkrellm_locale_dup_string(&fs->label, label, &fs->label_shadow);
-	fs->mount.directory =
-		g_strdup(gkrellm_gtk_entry_get_text(&(GTK_COMBO(dir_combo)->entry)));
+
+	entry = gtk_bin_get_child(GTK_BIN(dir_combo_box));
+	fs->mount.directory = g_strdup(gkrellm_gtk_entry_get_text(&entry));
+
 	if (show_button)
 		fs->show_if_mounted = GTK_TOGGLE_BUTTON(show_button)->active;
 	if (mounting_button)
@@ -2060,10 +2115,15 @@ cb_delete(GtkWidget *widget)
 static void
 cb_data_format(GtkWidget *widget, gpointer data)
 	{
-	GList	*list;
-	gchar	*s;
+	GList			*list;
+	FSmon			*fs;
+	GkrellmDecal	*d;
+	gchar			*s, buf[256];
+	gint			h_data, h_label;
+	GtkWidget *entry;
 
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(data_format_combo)->entry));
+	entry = gtk_bin_get_child(GTK_BIN(data_format_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
 
 	/* In case Pango markup tags, don't accept line unless valid markup.
 	|  Ie, markup like <span ...> xxx </span> or <b> xxx </b>
@@ -2075,8 +2135,35 @@ cb_data_format(GtkWidget *widget, gpointer data)
 		return;
 
 	if (gkrellm_locale_dup_string(&data_format, s, &data_format_locale))
+		{
 		for (list = fs_mon_list; list; list = list->next)
-			((FSmon *) list->data)->text_decal->value = -1;	/* Force redraw */
+			{
+			fs = (FSmon *) list->data;
+			fs->label_decal->value = -1;	/* Force redraw */
+
+			d = fs->data_decal;
+
+			format_fs_data(fs, data_format_locale, buf, sizeof(buf));
+			gkrellm_text_markup_extents(d->text_style.font, buf, strlen(buf),
+					NULL, &h_data, NULL, NULL);
+			h_data += d->text_style.effect;
+			h_label = fs->label_decal->h;
+
+			/* Rebuild fs mon panels if new format string height extents
+			|  are wrong for current label/data decals.
+			*/
+			if (   (d->h >= h_label && h_data != d->h)
+			    || (d->h < h_label && h_data >= h_label)
+			   )
+				{
+				sync_fs_panels();
+				break;
+				}
+			if (d->h < h_label && h_data < h_label && d->h != h_data)
+				gkrellm_move_decal(fs->panel, d, d->x,
+						d->y + (d->h - h_data) / 2);
+			}
+		}
 	}
 
 static void
@@ -2103,8 +2190,8 @@ cb_nfs_check_interval(GtkWidget *widget, GtkSpinButton *spin)
 	nfs_check_timeout = gtk_spin_button_get_value_as_int(spin);
 	}
 
-  /* Allow desination drops only on depth 2 paths and don't allow drops from
-  |  source depths of 1 (top level nodes).  Note: from some reason if I allow
+  /* Allow destination drops only on depth 2 paths and don't allow drops from
+  |  source depths of 1 (top level nodes).  Note: for some reason if I allow
   |  drops on depth 3 nodes (destination is on top of a second level node) I
   |  am not getting "drag_end" callbacks.
   */
@@ -2119,7 +2206,7 @@ row_drop_possible(GtkTreeDragDest *drag_dest, GtkTreePath *path,
 		return FALSE;
 	src_path = gtk_tree_row_reference_get_path(row_reference);
 	indices = gtk_tree_path_get_indices(path);
-//	printf("drop path: indices=[%d,%d]:%d, path=%s\n",
+//	g_debug("drop path: indices=[%d,%d]:%d, path=%s\n",
 //			indices[0], indices[1], gtk_tree_path_get_depth(path),
 //			gtk_tree_path_to_string(path));
 
@@ -2176,7 +2263,7 @@ create_fs_panels_page(GtkWidget *vbox)
 	GtkWidget				*scrolled;
 	GtkTreeModel			*model;
 	GtkCellRenderer			*renderer;
-	GList					*list, *combo_list;
+	GList					*list;
 
 
 	hbox = gtk_hbox_new(FALSE, 0);
@@ -2190,7 +2277,7 @@ create_fs_panels_page(GtkWidget *vbox)
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
 				GTK_SHRINK, GTK_SHRINK, 2, 1);
 	label_entry = gtk_entry_new();
-	gtk_entry_set_max_length(GTK_ENTRY(label_entry), 16);
+//	gtk_entry_set_max_length(GTK_ENTRY(label_entry), 16);
 	gtk_widget_set_size_request(label_entry, 100, -1);
 	gtk_table_attach_defaults(GTK_TABLE(table), label_entry, 1, 2, 0, 1);
 
@@ -2201,18 +2288,16 @@ create_fs_panels_page(GtkWidget *vbox)
 #endif
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
 				GTK_SHRINK, GTK_SHRINK, 2, 1);
-	dir_combo = gtk_combo_new();
-	gtk_table_attach_defaults(GTK_TABLE(table), dir_combo, 1, 2, 1, 2);
-	combo_list = NULL;
+	dir_combo_box = gtk_combo_box_entry_new_text();
+	gtk_table_attach_defaults(GTK_TABLE(table), dir_combo_box, 1, 2, 1, 2);
 	for (list = fstab_list; list; list = list->next)
-		combo_list = g_list_append(combo_list,
-					((Mount *)list->data)->directory);
-	gtk_combo_set_popdown_strings( GTK_COMBO(dir_combo), combo_list);
-	gtk_combo_set_case_sensitive(GTK_COMBO(dir_combo), TRUE);
-	g_list_free(combo_list);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dir_combo)->entry), "");
-	g_signal_connect(G_OBJECT(GTK_COMBO(dir_combo)->entry),
-				"changed", GTK_SIGNAL_FUNC (cb_combo_changed), NULL);
+		{
+		gtk_combo_box_append_text(GTK_COMBO_BOX(dir_combo_box),
+				((Mount *)list->data)->directory);
+		}
+	gtk_combo_box_set_active(GTK_COMBO_BOX(dir_combo_box), -1);
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(dir_combo_box)),
+			"changed", G_CALLBACK(cb_combo_changed), NULL);
 
 	vbox1 = gtk_vbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox1, FALSE, FALSE, 0);
@@ -2417,13 +2502,11 @@ N_("Shows and hides the secondary file system monitors.\n")
 };
 
 static void
-create_fs_tab(GtkWidget *tab_vbox)
+fs_tab_create(GtkWidget *tab_vbox)
 	{
 	GtkWidget		*tabs;
-	GtkWidget		*vbox, *vbox1, *hbox;
+	GtkWidget		*vbox, *vbox1;
 	GtkWidget		*text;
-	GtkWidget		*label;
-	GList			*list;
 	gint			i;
 
 	row_reference = NULL;
@@ -2445,7 +2528,9 @@ create_fs_tab(GtkWidget *tab_vbox)
 /* --Setup tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
 
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Options"), 4, FALSE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Options"),
+				4, 0, TRUE);
 	gkrellm_gtk_check_button_connected(vbox1, NULL,
 			binary_units, FALSE, FALSE, 0,
 			cb_binary_units, NULL,
@@ -2471,26 +2556,19 @@ create_fs_tab(GtkWidget *tab_vbox)
 			cb_nfs_check_interval, NULL, FALSE,
 			_("Seconds between data updates for remote mounted file systems"));
 		}
-	vbox1 = gkrellm_gtk_framed_vbox_end(vbox,
-			_("Format String for Panel Labels"), 4, FALSE, 0, 2);
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 2);
-	data_format_combo = gtk_combo_new();
-	gtk_box_pack_start(GTK_BOX(hbox), data_format_combo, TRUE, TRUE, 0);
-	list = NULL;
-	list = g_list_append(list, data_format);
-	list = g_list_append(list, DEFAULT_DATA_FORMAT);
-	list = g_list_append(list, _("$t - $u used"));
-	list = g_list_append(list, _("$t - $U"));
-	gtk_combo_set_popdown_strings(GTK_COMBO(data_format_combo), list);
-	gtk_combo_set_case_sensitive(GTK_COMBO(data_format_combo), TRUE);
-	g_list_free(list);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(data_format_combo)->entry),
-			data_format);
-	g_signal_connect(G_OBJECT(GTK_COMBO(data_format_combo)->entry), "changed",
+
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Format String for Panel Labels"),
+				4, 0, TRUE);
+	data_format_combo_box = gtk_combo_box_entry_new_text();
+	gtk_box_pack_start(GTK_BOX(vbox1), data_format_combo_box, FALSE, FALSE, 2);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(data_format_combo_box), data_format);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(data_format_combo_box), DEFAULT_DATA_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(data_format_combo_box), ALT1_DATA_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(data_format_combo_box), ALT2_DATA_FORMAT);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(data_format_combo_box), 0);
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(data_format_combo_box)), "changed",
 			G_CALLBACK(cb_data_format), NULL);
-	label = gtk_label_new(_("Format string for FS data display"));
-	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
 
 /* --Info tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Info"));
@@ -2509,13 +2587,13 @@ static GkrellmMonitor	monitor_fs =
 	{
 	N_("File System"),	/* Name, for config tab.	*/
 	MON_FS,				/* Id,  0 if a plugin		*/
-	create_fs,			/* The create function		*/
-	update_fs,			/* The update function		*/
-	create_fs_tab,		/* The config tab create function	*/
+	fs_create,			/* The create function		*/
+	fs_update,			/* The update function		*/
+	fs_tab_create,		/* The config tab create function	*/
 	NULL,				/* Instant config */
 
-	save_fs_config,		/* Save user conifg			*/
-	load_fs_config,		/* Load user config			*/
+	fs_config_save,		/* Save user conifg			*/
+	fs_config_load,		/* Load user config			*/
 	FS_CONFIG_KEYWORD,	/* config keyword			*/
 
 	NULL,				/* Undef 2	*/
