@@ -1,33 +1,51 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2010 Bill Wilson
 |
-|  Author:  Bill Wilson    bill@gkrellm.net
+|  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
+
+#ifndef GKRELLM_H
+#define GKRELLM_H
+
 #if defined(HAVE_CONFIG_H)
 #include "config.h"
-#endif  
+#endif
+#include "log.h"
 
 #if !defined(WIN32)
 #include <sys/param.h>
 #endif
 
 #include <gtk/gtk.h>
+#include <glib/gstdio.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,7 +57,8 @@
 #if !defined(WIN32)
 #include <unistd.h>
 #else
-#include "win32.h"
+    #include <winsock2.h>
+    #include <process.h>
 #endif
 
 #include <sys/stat.h>
@@ -52,7 +71,7 @@
 
 #if !defined(__FreeBSD__) && !defined(__linux__) && !defined(__NetBSD__) \
 	&& !defined(__OpenBSD__) && !defined(__solaris__) && !defined(WIN32) \
-	&& !defined(__APPLE__)
+	&& !defined(__APPLE__) && !defined(__DragonFly__)
 #define  USE_LIBGTOP
 #endif
 
@@ -79,7 +98,7 @@
 #	define gettext(String) (String)
 #	define dgettext(Domain,String) (String)
 #	define dcgettext(Domain,String,Type) (String)
-#	define bindtextdomain(Domain,Directory) (Domain) 
+#	define bindtextdomain(Domain,Directory) (Domain)
 #endif	/* ENABLE_NLS */
 
 
@@ -88,7 +107,7 @@
 /* -------------------------------------------------------------------
 */
 #define	GKRELLM_VERSION_MAJOR	2
-#define	GKRELLM_VERSION_MINOR	2
+#define	GKRELLM_VERSION_MINOR	3
 #define	GKRELLM_VERSION_REV		5
 #define	GKRELLM_EXTRAVERSION	""
 
@@ -125,14 +144,10 @@
 
 #else
 
-#define     LOCAL_THEMES_DIR		"./themes"
-#if !defined(SYSTEM_THEMES_DIR)
-#define     SYSTEM_THEMES_DIR		"./themes"
-#endif
-#define     LOCAL_PLUGINS_DIR		"./plugins"
-#if !defined(SYSTEM_PLUGINS_DIR)
-#define     SYSTEM_PLUGINS_DIR		"./plugins"
-#endif
+#undef      LOCAL_THEMES_DIR
+#undef      SYSTEM_THEMES_DIR
+#undef      LOCAL_PLUGINS_DIR
+#undef      SYSTEM_PLUGINS_DIR
 
 #endif
 
@@ -140,7 +155,7 @@
 #define	ON				1
 #define	OFF				0
 
-#define	CFG_BUFSIZE		384
+#define	CFG_BUFSIZE		512
 
 
   /* Features
@@ -268,7 +283,7 @@ typedef struct
 	GdkBitmap	*stencil;
 	gint		y_src;		/* Offset into pixmap if a vertical stack	*/
 	gint		w, h;		/* Size of the decal						*/
-	gint		x, y;		/* Position of decal in a drawable			*/	
+	gint		x, y;		/* Position of decal in a drawable			*/
 	gshort		flags,
 				state;
 	gint		value;		/* Index into stack, text value, etc	*/
@@ -1049,19 +1064,20 @@ typedef struct _GkrellmMonitor
 	void		(*save_user_config)(FILE *);
 	void		(*load_user_config)(gchar *);
 	gchar		*config_keyword;
-	
+
 	void		(*undef2)(void);
 	void		(*undef1)(void);
 	GkrellmMonprivate *privat;
 
 	gint		insert_before_id;		/* If plugin, insert before this mon*/
-	
+
 	void		*handle;				/* If monitor is a plugin.	*/
 	gchar		*path;					/* 	"						*/
 	}
 	GkrellmMonitor;
 
-extern GList	*gkrellm_monitor_list;
-
 
 #include "gkrellm-public-proto.h"
+
+#endif // GKRELLM_H
+
