@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2007 Bill Wilson
+|  Copyright (C) 1999-2010 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -17,6 +17,18 @@
 |
 |  You should have received a copy of the GNU General Public License
 |  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -756,7 +768,7 @@ create_about_tab(GtkWidget *vbox)
 	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
 
 	buf = g_strdup_printf(_("GKrellM %d.%d.%d%s\nGNU Krell Monitors\n\n"
-				"Copyright (c) 1999-2007 by Bill Wilson\n"
+				"Copyright (c) 1999-2010 by Bill Wilson\n"
 				"billw@gkrellm.net\n"
 				"http://gkrellm.net\n\n"
 				"Released under the GNU General Public License"),
@@ -815,7 +827,7 @@ create_about_tab(GtkWidget *vbox)
 	buf = g_strdup_printf(
 				_("Windows system dependent code was contributed by:\n"
 				"Bill Nalen <bill@nalens.com>\n"
-				"Stefan Gehn <metz81@web.de>\n"));
+				"Stefan Gehn <stefan+gkrellm@srcbox.net>\n"));
 	label = gtk_label_new(buf);
 	g_free(buf);
 	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
@@ -1302,7 +1314,7 @@ get_theme_author(gchar *path)
 	if (!path || *path == '\0')
 		return buf;
 	rcfile = g_strdup_printf("%s/%s", path, GKRELLMRC);
-	f = fopen(rcfile, "r");
+	f = g_fopen(rcfile, "r");
 	g_free(rcfile);
 	if (!f)
 		return buf;
@@ -1433,6 +1445,28 @@ gkrellm_make_themes_list(void)
 	theme_dir = gtk_rc_get_theme_dir();
 	add_themes_to_list(theme_dir, TRUE);
 
+#if defined(WIN32)
+	theme_dir = NULL;
+#if GLIB_CHECK_VERSION(2,16,0)
+	gchar *install_path;
+	install_path = g_win32_get_package_installation_directory_of_module(NULL);
+	if (install_path != NULL)
+		{
+		theme_dir = g_build_filename(install_path, "share", "gkrellm2", "themes", NULL);
+		g_free(install_path);
+		}
+#else
+	// deprecated since glib 2.16.0
+	path = g_win32_get_package_installation_subdirectory(NULL, NULL, "share/gkrellm2/themes");
+#endif
+	if (theme_dir)
+		{
+		add_themes_to_list(theme_dir, FALSE);
+		g_free(theme_dir);
+		theme_dir = NULL;
+		}
+#endif
+
 #if defined(LOCAL_THEMES_DIR)
 	add_themes_to_list(LOCAL_THEMES_DIR, FALSE);
 #endif
@@ -1558,7 +1592,7 @@ gkrellm_save_theme_config(void)
 	path = gkrellm_make_config_file_name(gkrellm_homedir(),
 					GKRELLM_THEME_CONFIG);
 
-	if ((f = fopen(path, "w")) != NULL)
+	if ((f = g_fopen(path, "w")) != NULL)
 		{
 		fprintf(f, "%s\n", _GK.theme_path);
 		fprintf(f, "%d\n", _GK.theme_alternative);
@@ -1590,14 +1624,13 @@ gkrellm_load_theme_config(void)
 		{
 		path = gkrellm_make_config_file_name(gkrellm_homedir(),
 					GKRELLM_THEME_CONFIG);
-		f = fopen(path, "r");
+		f = g_fopen(path, "r");
 		g_free(path);
 		if (f && fgets(buf, sizeof(buf), f))
 			{
 			if ((s = strchr(buf, (gint) '\n')) != NULL)
 				*s = '\0';
-			if (_GK.debug_level & DEBUG_GUI)
-				printf("gkrellm_load_theme_config: %s\n", buf);
+			gkrellm_debug(DEBUG_GUI, "gkrellm_load_theme_config: %s\n", buf);
 			s = buf;
 			if (s && *s != '#' && *s != '\0' && strcmp(s, "Default"))
 				{
@@ -1611,8 +1644,7 @@ gkrellm_load_theme_config(void)
 				{
 				if ((s = strchr(buf, (gint) '\n')) != NULL)
 					*s = '\0';
-				if (_GK.debug_level & DEBUG_GUI)
-					printf("gkrellm_load_theme_config: %s\n", buf);
+				gkrellm_debug(DEBUG_GUI, "gkrellm_load_theme_config: %s\n", buf);
 				if (i == 0)
 					sscanf(buf, "%d", &_GK.theme_alternative);
 				if (i == 1 && !strstr(buf, "*-*"))	/* XXX Trap out GdkFont */
@@ -1650,8 +1682,8 @@ gkrellm_read_theme_event(GtkSettings  *settings)
 					/* so get the current gtk theme name and switch to it  */
 		{
 		g_object_get(_GK.gtk_settings, "gtk-theme-name", &theme_name, NULL);
-		if ((_GK.debug_level & DEBUG_GUI) && theme_name)
-			printf("notify::gtk-theme-name: %s\n", theme_name);
+		if (theme_name)
+			gkrellm_debug(DEBUG_GUI, "notify::gtk-theme-name: %s\n", theme_name);
 
 		if (   gkrellm_dup_string(&_GK.gtk_theme_name, theme_name)
 			&& _GK.track_gtk_theme_name
@@ -2091,8 +2123,8 @@ create_config_page(GkrellmMonitor *mon, GtkTreeStore *tree, GtkTreeIter *iter,
 			PAGE_COLUMN, page,
 			-1);
 
-	if (_GK.debug_level & DEBUG_GUI)
-		printf("create_config_page %d: %s\n", page, mon ? mon->name : "--");
+	gkrellm_debug(DEBUG_GUI, "create_config_page %d: %s\n", page,
+		mon ? mon->name : "--");
 
 	return vbox;
 	}
@@ -2162,8 +2194,7 @@ gkrellm_remove_plugin_config_page(GkrellmMonitor *mon)
 			if (mon->privat->config_page >= tmon->privat->config_page)
 				continue;
 			tmon->privat->config_page -= 1;
-			if (_GK.debug_level & DEBUG_GUI)
-				printf("config_page %d: %s\n",
+			gkrellm_debug(DEBUG_GUI, "config_page %d: %s\n",
 						tmon->privat->config_page,  tmon->name);
 			}
 		}
@@ -2227,8 +2258,7 @@ cb_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
 	set_apply_mode(instant);
 
 	gtk_notebook_set_current_page(config_notebook, page);
-	if (_GK.debug_level & DEBUG_GUI)
-		printf("tree_selection_changed %d: %s\n",
+	gkrellm_debug(DEBUG_GUI, "tree_selection_changed %d: %s\n",
 					page, selected_monitor ? selected_monitor->name : "--");
 	}
 
