@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2009 Bill Wilson
+|  Copyright (C) 1999-2010 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -170,7 +170,7 @@ _log_active_port(ActiveTCP *tcp)
 	if (_GK.debug_level & DEBUG_INET)
 		{
 		ap = inet_ntoa(tcp->remote_addr);
-		g_print("inet  OO----->  %x %s:%x\n",
+		g_debug("inet  OO----->  %x %s:%x\n",
 				tcp->local_port, ap, tcp->remote_port);
 		}
 	active_tcp_list = g_list_prepend(active_tcp_list, (gpointer) new_tcp);
@@ -248,7 +248,7 @@ gkrellm_inet_client_divert(void (*read_tcp_func)())
 
 /* ======================================================================== */
 
-#define DUMP_IN(in,tag)	printf("%s: %s %s %ld %ld %ld %ld\n", \
+#define DUMP_IN(in,tag)	g_debug("%s: %s %s %ld %ld %ld %ld\n", \
 						tag, in->label0, in->label1, \
             			in->port0_0, in->port0_1, in->port1_0, in->port1_1);
 
@@ -560,7 +560,7 @@ update_inet(void)
 				if (_GK.debug_level & DEBUG_INET)
 					{
 					ap = inet_ntoa(tcp->remote_addr);
-					g_print("inet  XX----->  %x %s:%x\n",
+					g_debug("inet  XX----->  %x %s:%x\n",
 							tcp->local_port, ap, tcp->remote_port);
 					}
 				free_tcp_list = g_list_prepend(free_tcp_list, tcp);
@@ -650,9 +650,12 @@ get_connection_string_thread(void *data)
 	GList					*list;
 	ActiveTCP				*tcp;
 #if defined(INET6)
-	struct sockaddr_storage	ss;
-	struct sockaddr_in		*sin;
-	struct sockaddr_in6		*sin6;
+	union {
+	    struct sockaddr_storage	ss;
+	    struct sockaddr_in		sin;
+	    struct sockaddr_in6	sin6;
+	    struct sockaddr		sa;
+	} ss;
 	gint					salen, flag = 0;
 	gchar					hbuf[NI_MAXHOST];
 	gchar					buf[NI_MAXHOST + 10];
@@ -670,34 +673,32 @@ get_connection_string_thread(void *data)
 		{
 		tcp = (ActiveTCP *) list->data;
 #if defined(INET6)
-		memset(&ss, 0, sizeof(ss));
+		memset(&ss.ss, 0, sizeof(ss.ss));
 		switch (tcp->family)
 			{
 			case AF_INET:
-				sin = (struct sockaddr_in *)&ss;
 				salen = sizeof(struct sockaddr_in);
-				memcpy(&sin->sin_addr, &tcp->remote_addr, salen);
+				memcpy(&ss.sin.sin_addr, &tcp->remote_addr, salen);
 #if defined(SIN6_LEN)
-				sin->sin_len = salen;
+				ss.sin.sin_len = salen;
 #endif
-				sin->sin_family = tcp->family;
+				ss.sin.sin_family = tcp->family;
 				break;
 			case AF_INET6:
-				sin6 = (struct sockaddr_in6 *)&ss;
 				salen = sizeof(struct sockaddr_in6);
-				memcpy(&sin6->sin6_addr, &tcp->remote_addr6, salen);
+				memcpy(&ss.sin6.sin6_addr, &tcp->remote_addr6, salen);
 #if defined(SIN6_LEN)
-				sin6->sin6_len = salen;
+				ss.sin6.sin6_len = salen;
 #endif
-				sin6->sin6_family = tcp->family;
+				ss.sin6.sin6_family = tcp->family;
 				/* XXX: We should mention about
 				|  scope, too. */
 				break;
 			default:
 				continue;
 			}
-		if (getnameinfo((struct sockaddr *)&ss, salen,
-					hbuf, sizeof(hbuf), NULL, 0, flag))
+		if (getnameinfo(&ss.sa, salen,
+				hbuf, sizeof(hbuf), NULL, 0, flag))
 			continue;
 		remote_host = hbuf;
 #else
@@ -1104,12 +1105,14 @@ fix_ports(InetMon *in)
 
 	if (!*in->label0)
 		{
-		in->port0_1 = in->port0_1 = in->data0_is_range = 0;
+		in->port0_1 = 0;
+		in->data0_is_range = 0;
 		--cd_length;
 		}
 	if (!*in->label1)
 		{
-		in->port1_1 = in->port1_1 = in->data1_is_range = 0;
+		in->port1_1 = 0;
+		in->data1_is_range = 0;
 		--cd_length;
 		}
 	if (in->data0_is_range && (in->port0_1 < in->port0_0))
@@ -1323,7 +1326,7 @@ gkrellm_inet_save_data(void)
 		{
 		in = (InetMon *) list->data;
 		fname = make_inet_data_fname(in);
-		if ((f = fopen(fname, "w")) == NULL)
+		if ((f = g_fopen(fname, "w")) == NULL)
 			continue;
 
 		fputs("minute hour yday width\n", f);
@@ -1357,7 +1360,7 @@ gkrellm_inet_load_data(void)
 		{
 		in = (InetMon *) list->data;
 		fname = make_inet_data_fname(in);
-		if ((f = fopen(fname, "r")) == NULL)
+		if ((f = g_fopen(fname, "r")) == NULL)
 			{
 			gkrellm_reset_chart(in->chart);
 			draw_inet_chart(in);
