@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2009 Bill Wilson
+|  Copyright (C) 1999-2010 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -958,7 +958,7 @@ gkrellm_create_krell(GkrellmPanel *p, GkrellmPiximage *im, GkrellmStyle *style)
 
 	if (im == NULL || style == NULL)
 		{
-		printf(_("create_krell: NULL image or style\n"));
+		g_warning(_("create_krell: NULL image or style\n"));
 		exit(0);
 		}
 
