@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2007 Bill Wilson
+|  Copyright (C) 1999-2010 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -17,13 +17,27 @@
 |
 |  You should have received a copy of the GNU General Public License
 |  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
 #include "gkrellm-private.h"
 #include "gkrellm-sysdeps.h"
+#include "log-private.h"
 
-#if GTK_CHECK_VERSION(2,4,0)
+// GTK+-win32 automatically uses the application-icon
+#if GTK_CHECK_VERSION(2,4,0) && !defined(WIN32)
 #include "icon.xpm"
 #endif
 
@@ -133,17 +147,15 @@ load_font(gchar *font_string, PangoFontDescription **gk_font,
 
 	if (font_string)
 		font_desc = pango_font_description_from_string(font_string);
-	if (_GK.debug_level & DEBUG_GUI)
-		printf("load_font: %s %p\n", font_string, font_desc);
+	gkrellm_debug(DEBUG_GUI, "load_font: %s %p\n", font_string, font_desc);
 
 	if (!font_desc)
 		{
 		for (i = 0; !font_desc && i < N_FALLBACK_FONTS; ++i)
 			{
 			font_desc = pango_font_description_from_string(fallback_fonts[i]);
-			if (_GK.debug_level & DEBUG_GUI)
-				printf("load_font trying fallback: %s\n",
-							fallback_fonts[i]);
+			gkrellm_debug(DEBUG_GUI, "load_font trying fallback: %s\n",
+						fallback_fonts[i]);
 			}
 		}
 	if (*gk_font)
@@ -222,7 +234,7 @@ setup_colors()
 
 	/* Set up the depth 1 GCs
 	*/
-	/* printf("white pixel = %ld\n", _GK.white_color.pixel); */
+	/* g_print("white pixel = %ld\n", _GK.white_color.pixel); */
 	if (_GK.bit1_GC == NULL)
 		{
 		GdkBitmap	*dummy_bitmap;
@@ -267,21 +279,20 @@ set_or_save_position(gint save)
 			}
 		if (   !_GK.no_config
 		    && (_GK.x_position != x_last || _GK.y_position != y_last)
-			&& (f = fopen(path, "w")) != NULL
+			&& (f = g_fopen(path, "w")) != NULL
 		   )
 			{
 			x_last = _GK.x_position;
 			y_last = _GK.y_position;
 			fprintf(f, "%d %d\n", _GK.x_position, _GK.y_position);
 			fclose(f);
-			if (_GK.debug_level & DEBUG_POSITION)
-				printf("save_position: %d %d\n", x_last, y_last);
+			gkrellm_debug(DEBUG_POSITION, "save_position: %d %d\n", x_last, y_last);
 			}
 		save_position_countdown = 0;
 		}
 	else if (!_GK.withdrawn)  /* In slit conflicts with setting position */
 		{
-		if ((f = fopen(path, "r")) != NULL)
+		if ((f = g_fopen(path, "r")) != NULL)
 			{
 			x = y = 0;
 			fscanf(f, "%d %d", &x, &y);
@@ -295,8 +306,7 @@ set_or_save_position(gint save)
 				_GK.y_position = y_last = y;
 				_GK.position_valid = TRUE;
 				gdk_window_move(gtree.window->window, x, y);
-				if (_GK.debug_level & DEBUG_POSITION)
-					printf("startup_position moveto %d %d (valid)\n", x, y);
+				gkrellm_debug(DEBUG_POSITION, "startup_position moveto %d %d (valid)\n", x, y);
 				}
 			}
 		}
@@ -602,8 +612,7 @@ top_frame_button_release(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 	if (!no_transparency)
 		gkrellm_winop_apply_rootpixmap_transparency();
 	moving_gkrellm = FALSE;
-	if (_GK.debug_level & DEBUG_POSITION)
-		printf("gkrellm moveto: x_pos=%d y_pos=%d\n",
+	gkrellm_debug(DEBUG_POSITION, "gkrellm moveto: x_pos=%d y_pos=%d\n",
 				_GK.x_position, _GK.y_position);
 	}
 
@@ -636,7 +645,9 @@ top_frame_button_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 		}
 	gtk_window_present(GTK_WINDOW(top_window));
 
-	if (_GK.client_mode && gkrellm_client_server_connect_state() == 0)
+	if (   _GK.client_mode
+	    && gkrellm_client_server_connect_state() == DISCONNECTED
+	   )
 		gkrellm_client_mode_connect_thread();
 
 	if (!_GK.withdrawn)		/* Move window unless in the slit */
@@ -1179,7 +1190,7 @@ gkrellm_pack_side_frames(void)
 		{
 		rf_image = gtk_image_new_from_pixmap(gtree.frame_right_pixmap, NULL);
 		gtk_box_pack_start(GTK_BOX(gtree.right_vbox), rf_image,
-					FALSE, FALSE, 0);	
+					FALSE, FALSE, 0);
 		gtk_widget_show(rf_image);
 		}
 	else
@@ -1199,9 +1210,9 @@ gkrellm_pack_side_frames(void)
 			{
 			gdk_window_move(gtree.window->window,
 							_GK.x_position, _GK.y_position);
-			if (_GK.debug_level & DEBUG_POSITION)
-				printf("pack moveto %d %d=y_position (y_gkrell=%d y_bot=%d)\n",
-						_GK.x_position, _GK.y_position, y_gkrell, y_bottom);
+			gkrellm_debug(DEBUG_POSITION,
+				"pack moveto %d %d=y_position (y_gkrell=%d y_bot=%d)\n",
+				_GK.x_position, _GK.y_position, y_gkrell, y_bottom);
 			}
 		y_pack = -1;
 		}
@@ -1218,9 +1229,9 @@ gkrellm_pack_side_frames(void)
 			if (_GK.position_valid)
 				{
 				gdk_window_move(gtree.window->window, _GK.x_position, y_pack);
-				if (_GK.debug_level & DEBUG_POSITION)
-					printf("pack moveto %d %d=y_pack (y_gkrell=%d y_bot=%d)\n",
-							_GK.x_position, y_pack, y_gkrell, y_bottom);
+				gkrellm_debug(DEBUG_POSITION,
+					"pack moveto %d %d=y_pack (y_gkrell=%d y_bot=%d)\n",
+					_GK.x_position, y_pack, y_gkrell, y_bottom);
 				}
 			}
 		/* If GKrellM bottom edge was <= screen bottom, then move to make
@@ -1233,9 +1244,9 @@ gkrellm_pack_side_frames(void)
 			if (_GK.position_valid)
 				{
 				gdk_window_move(gtree.window->window, _GK.x_position, y_pack);
-				if (_GK.debug_level & DEBUG_POSITION)
-					printf("pack moveto %d %d=y_pack (y_gkrell=%d on_bottom)\n",
-							_GK.x_position, y_pack, y_gkrell);
+				gkrellm_debug(DEBUG_POSITION,
+					"pack moveto %d %d=y_pack (y_gkrell=%d on_bottom)\n",
+					_GK.x_position, y_pack, y_gkrell);
 				}
 			}
 		}
@@ -1293,8 +1304,8 @@ fix_edges()
 		gdk_window_move(gtree.window->window, x, y);
 		if (y != _GK.y_position)
 			y_pack = y;
-		if (_GK.debug_level & DEBUG_POSITION)
-			printf("fix_edges: %d %d (y_pos=%d)\n", x, y, _GK.y_position);
+		gkrellm_debug(DEBUG_POSITION,
+			"fix_edges: %d %d (y_pos=%d)\n", x, y, _GK.y_position);
 		}
 	}
 
@@ -1319,7 +1330,7 @@ gkrellm_render_spacer(GkrellmSpacer *spacer, gint y_src, gint h_src,
 	   )
 		{
 		if (spacer->height > 0)
-			fprintf(stderr, "Bad image size for spacer or bg_chart.\n");
+			g_warning("Bad image size for spacer or bg_chart.\n");
 		return FALSE;
 		}
 	piximage.border = spacer->piximage->border;
@@ -1486,14 +1497,14 @@ static void
 create_widget_tree()
 	{
 	gchar		*title;
-#if GTK_CHECK_VERSION(2,4,0)
+#if GTK_CHECK_VERSION(2,4,0) && !defined(WIN32)
 	GdkPixbuf	*icon;
 #endif
 
 	gtree.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_widget_set_name(gtree.window, PACKAGE);
 
-#if GTK_CHECK_VERSION(2,4,0)
+#if GTK_CHECK_VERSION(2,4,0) && !defined(WIN32)
 	icon = gdk_pixbuf_new_from_xpm_data((const gchar **) icon_xpm);
 	gtk_window_set_default_icon(icon);
 #endif
@@ -1573,7 +1584,7 @@ cb_client_event(GtkWidget  *widget, GdkEventClient *event, gpointer data)
 	if (!atom_gkrellm_read_theme)
 		atom_gkrellm_read_theme = gdk_atom_intern("_GKRELLM_READ_THEME",FALSE);
 
-	if (event->message_type == atom_gkrellm_read_theme) 
+	if (event->message_type == atom_gkrellm_read_theme)
 		gkrellm_read_theme_event(NULL);
 	return FALSE;
 	}
@@ -1673,12 +1684,16 @@ cb_configure_notify(GtkWidget *widget, GdkEventConfigure *ev, gpointer data)
 		if (y >= 0 && y < _GK.h_display - 5 && y != y_pack)
 			_GK.y_position = y;
 		_GK.position_valid = TRUE;
-		if (!moving_gkrellm && (_GK.debug_level & DEBUG_POSITION))
-			printf("configure-event: x_pos=%d y_pos=%d x=%d y=%d y_pack=%d\n",
-						_GK.x_position, _GK.y_position, x, y, y_pack);
+		if (!moving_gkrellm)
+			gkrellm_debug(DEBUG_POSITION,
+				"configure-event: x_pos=%d y_pos=%d x=%d y=%d y_pack=%d\n",
+				_GK.x_position, _GK.y_position, x, y, y_pack);
+		}
+	else
+		{
+		gkrellm_debug(DEBUG_POSITION,
+			"locked configure-event: x=%d y=%d\n", x, y);
 		}
-	else if (_GK.debug_level & DEBUG_POSITION)
-			printf("locked configure-event: x=%d y=%d\n", x, y);
 
 	if (size_change || position_change)
 		gkrellm_winop_update_struts();
@@ -1791,6 +1806,7 @@ N_("       --nolock             Allow multiple gkrellm instances.\n"),
 N_("   -p, --plugin plugin.so   While developing, load your plugin under test.\n"),
 N_("       --demo               Force enabling of many monitors so themers can\n"
    "                            see everything. All config saving is inhibited.\n"),
+   "   -l, --logfile path       Enable error/debugging to a log file.\n",
 N_("   -v, --version            Print GKrellM version number and exit.\n"),
 N_("   -d, --debug-level n      Turn debugging on for selective code sections.\n"),
 
@@ -1810,10 +1826,36 @@ N_("\ndebug-level numbers are (bitwise OR to debug multiple sections):\n"
 static void
 usage()
 	{
+#ifdef WIN32
+	GtkWidget *dialog, *content_area, *scrolled, *text_view;
+
+	dialog = gtk_dialog_new_with_buttons(_("GKrellM"), NULL, GTK_DIALOG_MODAL,
+		GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
+	content_area = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+
+	scrolled = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
+			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_widget_set_size_request(scrolled, 400, 300);
+	gtk_container_add(GTK_CONTAINER(content_area), scrolled);
+
+	text_view = gtk_text_view_new();
+	gtk_text_view_set_editable(GTK_TEXT_VIEW(text_view), FALSE);
+	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(text_view), FALSE);
+	gtk_container_add(GTK_CONTAINER(scrolled), text_view);
+
+	gkrellm_gtk_text_view_append_strings(text_view, usage_string,
+		sizeof(usage_string) / sizeof(gchar *));
+
+	gtk_widget_show_all(dialog);
+	gtk_dialog_run(GTK_DIALOG(dialog));
+	gtk_widget_destroy(dialog);
+#else
 	gint	i;
 
-	for (i = 0; i < sizeof(usage_string) / sizeof(gchar *); ++i)
-		g_print(_(usage_string[i]));
+	for (i = 0; i < (sizeof(usage_string) / sizeof(gchar *)); ++i)
+		g_print("%s", _(usage_string[i]));
+#endif
 	}
 
 
@@ -1948,26 +1990,11 @@ load_builtin_monitors()
 	add_builtin(gkrellm_init_uptime_monitor());
 	}
 
-
-void
-gkrellm_print(const gchar *string)
-	{
-	gchar	*s;
-
-	s = g_locale_from_utf8(string, -1, NULL, NULL, NULL);
-	if (s)
-		{
-		fputs(s, stdout);
-		g_free(s);
-		}
-	else
-		fputs(string, stdout);
-	}
-
 static void
 gkrellm_exit(gint exit_code)
 	{
 	gkrellm_sys_main_cleanup();
+	gkrellm_log_cleanup();
 	exit(exit_code);
 	}
 
@@ -2023,9 +2050,10 @@ gkrellm_abort(gint sig)
 		do_xmessage = FALSE;
 		}
 
+	// FIXME: xmessage is only available on X11
 	snprintf(buf, sizeof(buf), "xmessage gkrellm %s:  %s  (%s)", fault,
 				(mon && mon->name) ? mon->name : "", state);
-	printf("%s\n", buf + 9);
+	g_warning("%s\n", buf + 9);
 	if (do_xmessage)
 		g_spawn_command_line_async(buf, NULL);
 
@@ -2044,29 +2072,51 @@ setup_signal_handler(void)
 	signal(SIGTERM, _signal_quit);
 	}
 
+void
+gkrellm_sys_setup_connect(void (*setup_func)())
+	{
+	_GK.sys_setup_func = setup_func;
+	}
 
 gint
 main(gint argc, gchar **argv)
 	{
-	gint		i;
-	gchar		*s;
+	gint						i;
+	gchar						*s;
+	enum GkrellmConnectResult	connect_result;
+	GtkWidget					*dlg;
 
 	gkrellm_sys_main_init();
 
 #ifdef ENABLE_NLS
 	gtk_set_locale();
 #endif
-
 	g_thread_init(NULL);
-
 	gtk_init(&argc, &argv);		/* Will call gdk_init() */
-
+	gkrellm_log_init();
 	gtk_widget_push_colormap(gdk_rgb_get_colormap());
 
 #ifdef ENABLE_NLS
 #ifdef LOCALEDIR
+#if defined(WIN32)
+	gchar *install_path;
+	gchar *locale_dir;
+	// Prepend app install path to locale dir
+	install_path = g_win32_get_package_installation_directory_of_module(NULL);
+	if (install_path != NULL)
+		{
+	    locale_dir = g_build_filename(install_path, LOCALEDIR, NULL);
+		if (locale_dir != NULL)
+			{
+			bindtextdomain(PACKAGE, locale_dir);
+			g_free(locale_dir);
+			}
+	    g_free(install_path);
+		}
+#else
 	bindtextdomain(PACKAGE, LOCALEDIR);
-#endif
+#endif /* !WIN32 */
+#endif /* LOCALEDIR */
 	textdomain(PACKAGE);
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
 #endif	/* ENABLE_NLS */
@@ -2080,8 +2130,6 @@ main(gint argc, gchar **argv)
 	signal(SIGSEGV, gkrellm_abort);
 	signal(SIGABRT, gkrellm_abort);
 
-	g_set_print_handler(gkrellm_print);
-
 	for (i = 1; i < argc; ++i)
 		{
 		s = argv[i];
@@ -2128,6 +2176,8 @@ main(gint argc, gchar **argv)
 			_GK.no_config = TRUE;
 		else if ((!strcmp(s, "debug-level") || !strcmp(s, "d")) && i < argc-1)
 			_GK.debug_level = (gint) strtoul(argv[++i], NULL, 0);
+		else if ((!strcmp(s, "logfile") || !strcmp(s, "l")) && i < argc-1)
+			gkrellm_log_set_filename(argv[++i]);
 		else if (!strncmp(s, "debug", 5))
 			{
 			if (s[5] != '\0')
@@ -2137,8 +2187,10 @@ main(gint argc, gchar **argv)
 			}
 		else if (!strcmp(s, "nolock"))
 			_GK.nolock = TRUE;
-		else if (!strcmp(s, "without-libsensors"))
+		else if (!strcmp(s, "without-libsensors"))		/* temporary */
 			_GK.without_libsensors = TRUE;
+		else if (!strcmp(s, "use-acpi-battery"))		/* temporary */
+			_GK.use_acpi_battery = TRUE;
 		else if (!strcmp(s, "config-clean"))
 			{
 			_GK.config_clean = TRUE;
@@ -2150,7 +2202,7 @@ main(gint argc, gchar **argv)
 			_GK.test += 1;
 		else if (!strcmp(s, "version") || !strcmp(s, "v"))
 			{
-			printf("%s %d.%d.%d%s\n", PACKAGE, GKRELLM_VERSION_MAJOR,
+			g_print("%s %d.%d.%d%s\n", PACKAGE, GKRELLM_VERSION_MAJOR,
 					GKRELLM_VERSION_MINOR, GKRELLM_VERSION_REV,
 					GKRELLM_EXTRAVERSION);
 			exit(0);
@@ -2168,17 +2220,55 @@ main(gint argc, gchar **argv)
 			}
 		}
 
+	if (_GK.debug_level > 0)
+		g_debug("--- GKrellM %d.%d.%d ---\n", GKRELLM_VERSION_MAJOR,
+			GKRELLM_VERSION_MINOR, GKRELLM_VERSION_REV);
+
+	if (_GK.sys_setup_func)
+		(*_GK.sys_setup_func)(argc, argv);
+
 	_GK.w_display = gdk_screen_get_width(gdk_screen_get_default());
 	_GK.h_display = gdk_screen_get_height(gdk_screen_get_default());
 
-	if (_GK.debug_level)
-		printf("--- GKrellM %d.%d.%d ---\n", GKRELLM_VERSION_MAJOR,
-				GKRELLM_VERSION_MINOR, GKRELLM_VERSION_REV);
-	if (_GK.server && !gkrellm_client_mode_connect())
-		exit(0);
+	if (_GK.server)
+		{
+		connect_result = gkrellm_client_mode_connect();
+		while (connect_result == BAD_CONNECT)
+			{
+			gint	result;
+
+			dlg = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
+					GTK_MESSAGE_ERROR, GTK_BUTTONS_YES_NO,
+					"GKrellM cannot connect to server:\n"
+					"\t%s:%d\n\n"
+					"Do you want to retry?",
+					_GK.server, _GK.server_port);
+			result = gtk_dialog_run(GTK_DIALOG(dlg));
+			gtk_widget_destroy(dlg);
+			if (result == GTK_RESPONSE_YES)
+				connect_result = gkrellm_client_mode_connect();
+			else
+				break;
+			}
+		if (connect_result == BAD_SETUP)
+			{
+			dlg = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
+					GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+					"GKrellM cannot get initial setup from server:\n"
+					"\t\t%s:%d\n",
+					_GK.server, _GK.server_port);
+			gtk_dialog_run(GTK_DIALOG(dlg));
+			gtk_widget_destroy(dlg);
+			}
+		if (connect_result != GOOD_CONNECT)
+			{
+			gkrellm_exit(0);
+			return 0;
+			}
+		}
 
 	check_gkrellm_directories();
-	gkrellm_load_user_config(NULL, FALSE);		
+	gkrellm_load_user_config(NULL, FALSE);
 	decorated = (_GK.command_line_decorated || _GK.decorated);
 	if (   _GK.command_line_plugin || _GK.command_line_theme
 		|| _GK.debug_level > 0 || _GK.debug > 0 || _GK.nolock
@@ -2252,33 +2342,16 @@ main(gint argc, gchar **argv)
 
 	gkrellm_start_timer(_GK.update_HZ);
 	setup_signal_handler();
-	gtk_main ();
+	gtk_main();
+
 	gkrellm_save_all();
+
+	/* disconnect from gkrellm-server if we're a client */
+	if (_GK.server)
+		{
+		gkrellm_client_mode_disconnect();
+		}
 	gkrellm_exit(0);
 
 	return 0;
 	}
-
-
-#if defined(WIN32) && defined(_WINDOWS)
-int APIENTRY WinMain(HINSTANCE	hInstance,
-					HINSTANCE	hPrevInstance,
-					LPSTR		lpCmdLine,
-					int			nCmdShow)
-	{
-	int		argc;
-	gint	i, ret;
-	gchar	*s, *sm;
-	gchar	**argv;
-	gchar	*cmd;
-
-	// need to get exe too
-	cmd = GetCommandLine();
-	g_shell_parse_argv(cmd, &argc, &argv, NULL);
-
-	ret = main(argc, argv);
-    g_strfreev(argv);
-	return ret;
-	}
-#endif
-
