@@ -1,34 +1,46 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2010 Bill Wilson
 |
-|  Author:  Bill Wilson    bill@gkrellm.net
+|  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
 #include "gkrellm-sysdeps.h"
 #include "gkrellm-private.h"
 
-
 #define	PPP_LOCK_FILE	"LCK..modem"
 
-#define TIMER_TYPE_NONE	0
-#define TIMER_TYPE_PPP	1
-#define TIMER_TYPE_IPPP	2
+#define TIMER_TYPE_NONE		0
+#define TIMER_TYPE_PPP		1
+#define TIMER_TYPE_IPPP		2
+#define TIMER_TYPE_SERVER	3
 
 
 #define		N_DAY_STATS		31
@@ -80,7 +92,9 @@ typedef struct
 	GkrellmChartconfig	*chart_config;
 	gboolean			enabled,
 						chart_labels,
-						force_up;
+						force_up,
+						real;
+
 	gchar				*label;
 
 	GkrellmAlert		*alert;
@@ -158,7 +172,7 @@ strcmp_net_name(NetMon *net1, NetMon *net2)
 	gint	n, n1, n2, len;
 
 	for (s = net1->name; *s; ++s)
-		if (isdigit(*s))
+		if (isdigit((unsigned char)*s))
 			break;
 	if (!*s)
 		return strcmp(net1->name, net2->name);
@@ -290,6 +304,7 @@ gkrellm_net_routed_event(gchar *name, gboolean routed)
 	else
 		net->down_event = TRUE;
 	net->up = routed;
+	net->real = TRUE;
 	}
 
 void
@@ -303,6 +318,7 @@ gkrellm_net_assign_data(gchar *name, gulong rx, gulong tx)
 		net->up = TRUE;
 	net->rx = rx;
 	net->tx = tx;
+	net->real = TRUE;
 	}
 
 void
@@ -385,6 +401,14 @@ gkrellm_net_led_positions(gint *x_rx_led, gint *y_rx_led,
 #include    "pixmaps/timer/bg_timer.xpm"
 #include    "pixmaps/timer/decal_timer_button.xpm"
 
+/* ISO 8601 date and month format strings for network stats gui*/
+#ifdef WIN32
+#define GK_NET_ISO_DATE "%Y-%m-%d"
+#define GK_NET_MONTH_STAT_DATE "%b %d"
+#else
+#define GK_NET_ISO_DATE "%F"
+#define GK_NET_MONTH_STAT_DATE "%b %e"
+#endif
 
 #define	MIN_GRID_RES		5
 #define	MAX_GRID_RES		100000000
@@ -452,7 +476,7 @@ static gint			timer_button_enabled;
 static gint			last_time = -1;
 
 
-static gint			timer_button_type,
+static gint			timer_button_type = TIMER_TYPE_NONE,
 					timer_button_state,
 					timer_button_old_state,
 					last_timer_command;
@@ -496,6 +520,13 @@ static gchar	days_in_month[12] =
 		{ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31  };
 
 
+  /* Called from client.c
+  */
+void
+gkrellm_net_server_has_timer(void)
+	{
+	timer_button_type = TIMER_TYPE_SERVER;
+	}
 
 static void
 draw_led(NetMon *net, int rxtx, int led_index)
@@ -743,6 +774,7 @@ get_connect_state(void)
 	switch (timer_button_type)
 		{
 		case TIMER_TYPE_NONE:
+		case TIMER_TYPE_SERVER:
 			break;
 		case TIMER_TYPE_PPP:
 			if (net_timed->up)
@@ -751,7 +783,7 @@ get_connect_state(void)
 				{
 				snprintf(buf, sizeof(buf),
 					 "%s/%s", lock_directory, PPP_LOCK_FILE);
-				if (stat(buf, &st) == 0)
+				if (g_stat(buf, &st) == 0)
 					state = TB_STANDBY;
 				else
 					{
@@ -760,7 +792,7 @@ get_connect_state(void)
 					*/
 					snprintf(buf, sizeof(buf), "%s/%s/%s",
 								gkrellm_homedir(), GKRELLM_DIR, PPP_LOCK_FILE);
-					if (stat(buf, &st) == 0)
+					if (g_stat(buf, &st) == 0)
 						state = TB_STANDBY;
 					}
 				}
@@ -787,10 +819,11 @@ get_connect_time(void)
 	switch (timer_button_type)
 		{
 		case TIMER_TYPE_NONE:
+		case TIMER_TYPE_SERVER:
 			break;
 		case TIMER_TYPE_PPP:
 			snprintf(buf, sizeof(buf), "/var/run/%s.pid", timer_button_iface);
-			if (stat(buf, &st) == 0)
+			if (g_stat(buf, &st) == 0)
 				t = st.st_mtime;
 			break;
 		case TIMER_TYPE_IPPP:
@@ -864,6 +897,12 @@ update_timer_button_monitor(void)
 			if (timer_button_state != TB_NORMAL)
 				draw_timer(timer_panel, (int) (time(0) - net_timer0), 0);
 			break;
+
+		case TIMER_TYPE_SERVER:
+			/* Button state is independent of displayed timer
+			*/
+			draw_timer(timer_panel, gkrellm_client_server_get_net_timer(), 0);
+			break;
 		}
 	if (net_timed && net_timed->up)
 		{
@@ -880,7 +919,7 @@ stale_pppd_files_debug(void)
 	gchar	buf[256];
 
 	snprintf(buf, sizeof(buf), "/var/run/%s.pid", timer_button_iface);
-	if (stat(buf, &st) == 0 && !net_timed->up)
+	if (g_stat(buf, &st) == 0 && !net_timed->up)
 		g_print(_("  **** Stale pppd pppX.pid file detected!\n"));
 	}
 
@@ -912,13 +951,14 @@ cb_timer_button_release(GtkWidget *widget, GdkEventButton *ev)
 
 	if (timer_button_state != TB_PRESSED)
 		return FALSE;
-	set_timer_button_state(save_tb_state);	
+	set_timer_button_state(save_tb_state);
 	if (! in_button(DECAL(timer_panel), ev))
 		return FALSE;
 
 	switch (timer_button_type)
 		{
 		case TIMER_TYPE_NONE:
+		case TIMER_TYPE_SERVER:
 			if (timer_button_state == TB_NORMAL)
 				{
 				if (*timer_on_command != '\0')
@@ -1001,12 +1041,18 @@ timed_net_visibility(void)
 	if ((net->force_up || net_is_up) && !chart_is_visible)
 		gkrellm_chart_show(net->chart, TRUE);	/* Make sure panel is shown */
 	else if (!net->force_up && !net_is_up && chart_is_visible)
+		{
 		gkrellm_chart_hide(net->chart, FALSE);  /* Don't hide the panel */
 
-	if (!net_is_up && chart_is_visible)
+		/* Save data whenever net has transitioned to a "not up" state.
+		|  Don't have the info to do this when there's a force_up and in
+		|  this case just rely on the every six hour tick data save.
+		*/
 		gkrellm_net_save_data();
+		}
 	}
 
+
 static gint
 net_expose_event(GtkWidget *widget, GdkEventExpose *ev)
 	{
@@ -1144,7 +1190,7 @@ static void
 format_net_data(NetMon *net, gchar *src_string, gchar *buf, gint size)
 	{
 	GkrellmChart	*cp;
-	gchar			c, *s, *s1;
+	gchar			c, *s, *s1, *result;
 	gint			len, bytes;
 	gdouble			fbytes;
 	gboolean		month, day, week;
@@ -1153,10 +1199,15 @@ format_net_data(NetMon *net, gchar *src_string, gchar *buf, gint size)
 		return;
 	--size;
 	*buf = '\0';
+	result = buf;
+
 	if (!src_string)
 		return;
 	cp = net->chart;
 	net->totals_shown = FALSE;
+
+	gkrellm_debug(DEBUG_CHART_TEXT, "net chart text: %s\n", src_string);
+
 	for (s = src_string; *s != '\0' && size > 0; ++s)
 		{
 		len = 1;
@@ -1229,6 +1280,9 @@ format_net_data(NetMon *net, gchar *src_string, gchar *buf, gint size)
 				len = snprintf(buf, size, "%s", net->name);
 			else if (c == 'H')
 				len = snprintf(buf, size, "%s", gkrellm_sys_get_host_name());
+			else if ((s1 = gkrellm_plugin_get_exported_label(mon_net, c,
+						net->name)) != NULL)
+				len = snprintf(buf, size, "%s", s1);
 			else
 				{
 				*buf = *s;
@@ -1259,7 +1313,9 @@ format_net_data(NetMon *net, gchar *src_string, gchar *buf, gint size)
 		size -= len;
 		buf += len;
 		}
-	*buf = '\0';	
+	*buf = '\0';
+
+	gkrellm_debug(DEBUG_CHART_TEXT, "              : %s\n", result);
 	}
 
 static void
@@ -1562,7 +1618,7 @@ days_in_year(gint y)
 
   /* Adjust a tm date structure to reference its next day.
   */
-static void 
+static void
 next_day(struct tm *t)
 	{
 	if ((t->tm_wday += 1) > SATURDAY)
@@ -1616,7 +1672,7 @@ gkrellm_net_save_data(void)
 			continue;
 		snprintf(fname, sizeof(fname), "%s%c%s", net_data_dir,
 					G_DIR_SEPARATOR, net->name);
-		if ((f = fopen(fname, "w")) == NULL)
+		if ((f = g_fopen(fname, "w")) == NULL)
 			continue;
 		fprintf(f, "%d\n", NET_DATA_VERSION);
 		fputs("wday mday month yday year\n", f);
@@ -1664,13 +1720,13 @@ net_stat_set_date_string(NetStat *ns, enum StatType stat_type, struct tm *t)
 
 	if (stat_type == DAY_STAT)
 		{
-		strftime(buf, sizeof(buf), "%F", &tm);	/* %Y-%m-%d (the ISO 8601) */
+		strftime(buf, sizeof(buf), GK_NET_ISO_DATE, &tm);
 		}
 	else if (stat_type == WEEK_STAT)
 		{
 		while (tm.tm_wday != SATURDAY)
 			next_day(&tm);
-		strftime(buf, sizeof(buf), "%F", &tm);
+		strftime(buf, sizeof(buf), GK_NET_ISO_DATE, &tm);
 		}
 	else if (stat_type == MONTH_STAT)
 		{
@@ -1703,8 +1759,8 @@ net_stat_set_date_string(NetStat *ns, enum StatType stat_type, struct tm *t)
 				tmx.tm_mday = days_in_month[tmx.tm_mon];
 			if (tm.tm_mday > days_in_month[tm.tm_mon])
 				tm.tm_mday = days_in_month[tm.tm_mon];
-			strftime(bufa, sizeof(bufa), "%b %e", &tmx);
-			strftime(bufb, sizeof(bufb), "%b %e", &tm);
+			strftime(bufa, sizeof(bufa), GK_NET_MONTH_STAT_DATE, &tmx);
+			strftime(bufb, sizeof(bufb), GK_NET_MONTH_STAT_DATE, &tm);
 			snprintf(buf, sizeof(buf), "%s - %s", bufa, bufb);
 			}
 		}
@@ -1766,7 +1822,7 @@ load_net_data(void)
 		net = (NetMon *) list->data;
 		snprintf(fname, sizeof(fname), "%s%c%s", net_data_dir,
 					G_DIR_SEPARATOR, net->name);
-		if ((f = fopen(fname, "r")) == NULL)
+		if ((f = g_fopen(fname, "r")) == NULL)
 			continue;
 		fgets(buf, sizeof(buf), f);
 		if (sscanf(buf, "%d\n", &version) != 1)
@@ -1816,7 +1872,7 @@ load_net_data(void)
 						&net->week_stats[0].connect_time,
 						&net->month_stats[0].connect_time);
 				}
-			
+
 			}
 		else if (version == NET_DATA_VERSION)
 			{
@@ -1951,7 +2007,7 @@ create_net_timer(GtkWidget *vbox, gint first_create)
 	else
 		{
 		/* time_decal and button_decal are initially at same y = top_margin
-		*/	
+		*/
 		if (time_decal->h > button_decal->h)
 			button_decal->y += (time_decal->h - button_decal->h) / 2;
 		else
@@ -2033,7 +2089,7 @@ update_net(void)
 			if (net->up && !net->up_prev)
 				net->up_event = TRUE;
 			else if (!net->up && net->up_prev)
-				net->down_event = TRUE; 
+				net->down_event = TRUE;
 			}
 		}
 	for (list = net_mon_list; list; list = list->next)
@@ -2150,6 +2206,9 @@ create_timed_monitor(void)
 	NetMon		*net;
 	TimerType	*tt;
 
+	if (_GK.client_mode)
+		return;
+
 	net_timed = NULL;
 	timer_button_type = TIMER_TYPE_NONE;
 	time(&net_timer0);
@@ -2228,7 +2287,7 @@ load_net_extra_piximages(void)
 	|      THEME_DIR/timer/bg_timer.png
 	|  and for a border for it from the gkrellmrc in the format:
 	|      set_piximage_border timer_bg_timer l,r,t,b
-	| There is no default for bg_timer, ie it may end up being NULL. 
+	| There is no default for bg_timer, ie it may end up being NULL.
 	*/
 	if (!bg_timer_style)		/* Used just for the bg_timer border */
 		bg_timer_style = gkrellm_style_new0();
@@ -2379,6 +2438,8 @@ save_net_config(FILE *f)
 	for (list = net_mon_list; list; list = list->next)
 		{
 		net = (NetMon *) list->data;
+		if (!net->enabled  && !net->real)
+			continue;
 		fprintf(f, "%s enables %s %d %d %d\n", NET_CONFIG_KEYWORD, net->name,
 				net->enabled, net->chart_labels, net->force_up);
 		gkrellm_save_chartconfig(f, net->chart_config,
@@ -2401,7 +2462,9 @@ save_net_config(FILE *f)
 						net->alert_uses_rx, net->alert_uses_tx);
 			}
 		}
-	fprintf(f, "%s timer_enabled %d\n", NET_CONFIG_KEYWORD,
+
+	if (!_GK.client_mode || timer_button_type != TIMER_TYPE_SERVER)
+		fprintf(f, "%s timer_enabled %d\n", NET_CONFIG_KEYWORD,
 						timer_button_enabled);
 	fprintf(f, "%s timer_seconds %d\n", NET_CONFIG_KEYWORD, timer_seconds);
 	if (!_GK.client_mode)
@@ -2428,9 +2491,11 @@ load_net_config(gchar *arg)
 	if (n != 2)
 		return;
 
-	if (!strcmp(config, "timer_enabled") && ! _GK.demo)
-		sscanf(item, "%d", &timer_button_enabled);
-	else if (!strcmp(config, "timer_seconds"))
+	if (!_GK.client_mode || timer_button_type != TIMER_TYPE_SERVER)
+		if (!strcmp(config, "timer_enabled") && ! _GK.demo)
+			sscanf(item, "%d", &timer_button_enabled);
+
+	if (!strcmp(config, "timer_seconds"))
 		sscanf(item, "%d", &timer_seconds);
 	else if (!strcmp(config, "reset_mday"))
 		sscanf(item, "%d", &reset_mday);
@@ -2455,7 +2520,7 @@ load_net_config(gchar *arg)
 			{	/* Can't get resolution, label, launch or tooltip */
 			sscanf(item1, "%*d %d %*s %d %d", &enable, &ch_labels, &force);
 			strcpy(config, "enables");
-			sprintf(item1, "%d %d %d", enable, ch_labels, force);
+			snprintf(item1, sizeof(item1), "%d %d %d", enable, ch_labels, force);
 			}
 		/* Remaining configs will have a net name
 		*/
@@ -2497,8 +2562,8 @@ load_net_config(gchar *arg)
 
 static GtkWidget	*pon_entry,
 					*poff_entry,
-					*timer_iface_combo,
-					*text_format_combo;
+					*timer_iface_combo_box,
+					*text_format_combo_box;
 
 static GtkWidget	*enable_net_timer_button;
 static GtkWidget	*net_timer_seconds_button;
@@ -2548,9 +2613,8 @@ cb_enable(GtkWidget *button, NetMon *net)
 			}
 		}
 	gtk_widget_set_sensitive(net->alert_button, net->enabled);
-	if (net == net_timed && !net->enabled && timer_iface_combo)
-		gtk_entry_set_text(
-					GTK_ENTRY(GTK_COMBO(timer_iface_combo)->entry), "none");
+	if (net == net_timed && !net->enabled && timer_iface_combo_box)
+		gtk_combo_box_set_active(GTK_COMBO_BOX(timer_iface_combo_box), 0);
 	else
 		sync_chart(net);
 	enable_in_progress = FALSE;
@@ -2571,8 +2635,10 @@ cb_text_format(GtkWidget *widget, gpointer data)
 	GList   *list;
 	NetMon	*net;
 	gchar   *s;
+	GtkWidget *entry;
 
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(text_format_combo)->entry));
+	entry = gtk_bin_get_child(GTK_BIN(text_format_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
 	gkrellm_locale_dup_string(&text_format, s, &text_format_locale);
 	for (list = net_mon_list; list; list = list->next)
 		{
@@ -2593,7 +2659,7 @@ cb_label_entry(GtkWidget *widget, NetMon *net)
 static void
 cb_set_alert(GtkWidget *button, NetMon *net)
 	{
-	
+
 #if 0
 	GtkTreeModel	*model;
 	GtkTreePath		*path;
@@ -2694,12 +2760,12 @@ cb_timer_enable(GtkWidget *button, gpointer data)
 				GTK_TOGGLE_BUTTON(enable_net_timer_button)->active,
 				&timer_button_enabled);
 	gtk_widget_set_sensitive(net_timer_seconds_button, timer_button_enabled);
-	if (timer_iface_combo)
+	if (timer_iface_combo_box)
 		{
+		/* Reset to "none" combo box entry */
 		if (!timer_button_enabled)
-			gtk_entry_set_text(
-					GTK_ENTRY(GTK_COMBO(timer_iface_combo)->entry), "none");
-		gtk_widget_set_sensitive(timer_iface_combo, timer_button_enabled);
+			gtk_combo_box_set_active(GTK_COMBO_BOX(timer_iface_combo_box), 0);
+		gtk_widget_set_sensitive(timer_iface_combo_box, timer_button_enabled);
 		}
 	net_timer_visibility();
 	}
@@ -2717,9 +2783,11 @@ static void
 cb_timer_iface(GtkWidget *widget, gpointer data)
 	{
 	gchar	*s;
+	GtkWidget *entry;
 
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(timer_iface_combo)->entry));
-	if (*s == '\0')
+	entry = gtk_bin_get_child(GTK_BIN(timer_iface_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
+	if (*s == '\0' || strcmp(s, _("none")) == 0)
 		s = "none";
 	if (gkrellm_dup_string(&timer_button_iface, s))
 		{
@@ -2887,36 +2955,38 @@ create_net_tab(GtkWidget *tab_vbox)
 
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Timer Button"));
 
-	gkrellm_gtk_check_button_connected(vbox, &enable_net_timer_button,
-				timer_button_enabled, FALSE, FALSE, 10,
-				cb_timer_enable, NULL,
-				_("Enable timer button"));
+	if (timer_button_type != TIMER_TYPE_SERVER) /* enabled by gkrellmd.conf? */
+		gkrellm_gtk_check_button_connected(vbox, &enable_net_timer_button,
+					timer_button_enabled, FALSE, FALSE, 10,
+					cb_timer_enable, NULL,
+					_("Enable timer button"));
+
 	gkrellm_gtk_check_button_connected(vbox, &net_timer_seconds_button,
 				timer_seconds, FALSE, FALSE, 0,
 				cb_timer_seconds, NULL,
 				_("Show seconds"));
+	gtk_widget_set_sensitive(net_timer_seconds_button, timer_button_enabled);
 	hbox = gtk_hbox_new (FALSE, 3);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox), 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 50);
 
 	if (!_GK.client_mode)
 		{
-		timer_iface_combo = gtk_combo_new();
-		gtk_box_pack_start(GTK_BOX(hbox), timer_iface_combo, FALSE, TRUE, 0);
-		list = NULL;
-		list = g_list_append(list, _("none"));
+		timer_iface_combo_box = gtk_combo_box_entry_new_text();
+		gtk_box_pack_start(GTK_BOX(hbox), timer_iface_combo_box, TRUE, TRUE, 0);
+		gtk_widget_set_sensitive(timer_iface_combo_box, timer_button_enabled);
+		gtk_combo_box_append_text(GTK_COMBO_BOX(timer_iface_combo_box),
+			_("none"));
 		for (tlist = timer_defaults_list; tlist; tlist = tlist->next)
 			{
 			tt = (TimerType *) tlist->data;
-			list = g_list_append(list, tt->name);
+			gtk_combo_box_append_text(GTK_COMBO_BOX(timer_iface_combo_box),
+				tt->name);
 			}
-		gtk_combo_set_popdown_strings(GTK_COMBO(timer_iface_combo), list);
-		gtk_combo_set_case_sensitive(GTK_COMBO(timer_iface_combo), TRUE);
-		g_list_free(list);
-		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(timer_iface_combo)->entry),
-					timer_button_iface);
-		g_signal_connect(G_OBJECT(GTK_COMBO(timer_iface_combo)->entry),
-					"changed", G_CALLBACK(cb_timer_iface), NULL);
+		gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(
+			GTK_BIN(timer_iface_combo_box))), timer_button_iface);
+		g_signal_connect(G_OBJECT(timer_iface_combo_box), "changed",
+			G_CALLBACK(cb_timer_iface), NULL);
 
 		label = gtk_label_new(_("Interface to link to the timer button"));
 		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
@@ -2985,12 +3055,13 @@ create_net_tab(GtkWidget *tab_vbox)
 		gtk_entry_set_text(GTK_ENTRY(net->label_entry), net->label);
 		gtk_box_pack_start (GTK_BOX (hbox), net->label_entry, FALSE, TRUE, 2);
 		label = gtk_label_new(_("Optional label for this interface."));
-		gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 4);
+		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 4);
 		g_signal_connect(G_OBJECT(net->label_entry), "changed",
 					G_CALLBACK(cb_label_entry), net);
 
-		vbox = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),
-						4, FALSE, 0, 2);
+		vbox = gkrellm_gtk_category_vbox(vbox,
+					_("Launch Commands"),
+					4, 0, TRUE);
 		table = gkrellm_gtk_launcher_table_new(vbox, 1);
 		gkrellm_gtk_config_launcher(table, 0,  &net->launch_entry,
 						&net->tooltip_entry, net->name, &net->launch);
@@ -3002,29 +3073,27 @@ create_net_tab(GtkWidget *tab_vbox)
 
 /* -- Setup tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Format String for Chart Labels"),
-			 4, FALSE, 0, 2);
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
-	text_format_combo = gtk_combo_new();
-	gtk_widget_set_size_request (GTK_WIDGET(text_format_combo), 300, -1);
-	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo, FALSE, TRUE, 0);
-	list = NULL;
-	list = g_list_append(list, text_format);
-	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
-	list = g_list_append(list, "\\c\\f$M\\n$T\\b\\c\\f$L");
-	list = g_list_append(list,
+
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Format String for Chart Labels"),
+				4, 0, TRUE);
+	text_format_combo_box = gtk_combo_box_entry_new_text();
+	gtk_widget_set_size_request (GTK_WIDGET(text_format_combo_box), 300, -1);
+	gtk_box_pack_start(GTK_BOX(vbox1), text_format_combo_box, FALSE, FALSE, 2);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box), text_format);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		DEFAULT_TEXT_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		"\\c\\f$M\\n$T\\b\\c\\f$L");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
 		_("\\f\\ww\\c\\f$M\\n\\f\\at\\.$t\\n\\f\\ar\\.$r\\b\\c\\f$L"));
-	list = g_list_append(list,
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
 		_("\\f\\ww\\c\\f$M\\n\\f\\at\\.$o\\n\\f\\ar\\.$i\\b\\c\\f$L"));
-	list = g_list_append(list,
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
 		_("\\f\\ww\\c\\f$M\\D2\\f\\ar\\.$r\\D1\\f\\at\\.$t\\b\\c\\f$L"));
-	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
-	gtk_combo_set_case_sensitive(GTK_COMBO(text_format_combo), TRUE);
-	g_list_free(list);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
-			text_format);
-	g_signal_connect(G_OBJECT(GTK_COMBO(text_format_combo)->entry), "changed",
+	gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(text_format_combo_box))),
+		text_format);
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(text_format_combo_box)), "changed",
 			G_CALLBACK(cb_text_format), NULL);
 
 	gkrellm_gtk_spin_button(vbox, NULL, (gfloat) reset_mday,
@@ -3086,10 +3155,13 @@ gkrellm_init_net_monitor(void)
 		}
 	else
 		{
-		timer_button_type = TIMER_TYPE_NONE;
 		timer_button_iface = g_strdup("none");
 		}
-	timer_button_enabled = !_GK.client_mode;
+	if (!_GK.client_mode)
+		timer_button_enabled = TRUE;
+	else
+		timer_button_enabled = (timer_button_type == TIMER_TYPE_SERVER);
+
 	timer_seconds = TRUE;
 	timer_on_command = g_strdup("");
 	timer_off_command = g_strdup("");
@@ -3113,7 +3185,7 @@ static GkrellmMonitor	monitor_timer =
 	NULL,				/* The update function		*/
 	NULL,				/* The config tab create function	*/
 	NULL,				/* Apply the config function		*/
-	
+
 	NULL,				/* Save user conifg			*/
 	NULL,				/* Load user config			*/
 	NULL,				/* config keyword			*/
