@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2009 Bill Wilson
+|  Copyright (C) 1999-2010 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -383,7 +383,7 @@ gkrellm_winop_options(gint argc, gchar **argv)
 		&& !_gkrellm_get_lock()
 	   )
 		{
-		printf("gkrellm: %s\n",
+		g_message("gkrellm: %s\n",
 			_("Exiting because multiple instances option is off.\n"));
 		exit(0);
 		}
@@ -481,8 +481,7 @@ gkrellm_winop_place_gkrellm(gchar *geom)
 	_GK.y_position = y;
 	_GK.x_position = x;
 	_GK.position_valid = TRUE;
-	if (_GK.debug_level & DEBUG_POSITION)
-		printf("geometry moveto %d %d\n", x, y);
+	gkrellm_debug(DEBUG_POSITION, "geometry moveto %d %d\n", x, y);
 	}
 
 void
